/*
//-----------------------------------------------------------------------------
OSS - OneTimePad V1.0 (C) 2003 by A. Ogut. All rights reserved. 

License to copy and use this software is granted provided that it 
is identified as the "OSS - OneTimePad" in all material mentioning 
or referencing this software or this function. 

License is also granted to make and use derivative works provided 
that such works are identified as "derived from the OSS - OneTimePad" 
in all material mentioning or referencing the derived work. 

OSS - Ogut Software Systems makes no representations concerning either 
the merchantability of this software or the suitability of this 
software for any particular purpose. It is provided "as is" 
without express or implied warranty of any kind. 

These notices must be retained in any copies of any part of this 
documentation and/or software. 
//-----------------------------------------------------------------------------
*/


// AsciiLut.c : Bitstuffing.
//
//-----------------------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>

#include "Global.h"
#include "Memalloc.h"
#include "AsciiLut.h"
#include "Longmath.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//-----------------------------------------------------------------------------
// Ascii 8 Bit -> 6 Bit (ber Lut)
// Lut mischen (Primary-Key) + Inverse Lut erzeugen


//static char Lut0[64] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/"


// Data -> cipher -> 6-Bit-Code stuffing -> Lut[6-Bit] -> printable-code

// printablecode -> INV-Lut[6-Bit] ->8-Bit-Code stuffing -> decipher -> Data


//-----------------------------------------------------------------------------
// Initialisiert eine Codetabelle (druckbare Zeichen)
// Lut: Zeiger auf Look-Up-Tabelle (Lut)
void Init6BitLut(char Lut[64])
{
  // Lut = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/"
  int i, n = 0;

  for (i = 'A'; i <= 'Z'; i++) 
    Lut[n++] = i;

  for (i = 'a'; i <= 'z'; i++) 
    Lut[n++] = i;

  for (i = '0'; i <= '9'; i++) 
    Lut[n++] = i;

  // brige Codes initialisieren
  Lut[n++] = '+';
  Lut[n++] = '/';
}


//-----------------------------------------------------------------------------
// Erstellt inverse Codetabelle
// Lut: Zeiger auf Look-Up-Tabelle, von der die Inverse erstellt werden soll.
// InvLut: Zeiger auf Inverse Look-Up-Tabelle (256 Zeichen!)
void Invers6BitLut(char Lut[64], char InvLut[256])
{
  int i;

  // Tabelle initialisieren
  for (i=0; i < 256; i++) 
    InvLut[i] = (char)-1;
  
  // Inverse Lut bilden
  for (i=0; i < 64; i++)
    InvLut[Lut[i]] = i;

}


//-----------------------------------------------------------------------------
// Look-Up-Tabelle mischen
// rnd:   Zeiger auf Zufallsvariable
// nPass: Anzahl der Codevertauschungen
// Lut:   Zeiger auf Look-Up-Tabelle
void Mix6BitLut(long *rnd, int nPass, char Lut[64])
{
  int i,j,k;
  char ch;

  for (i = 0; i < nPass; i++)
  {
    j = LongRandom(rnd, 64);
    k = LongRandom(rnd, 64);

    ch     = Lut[j];
    Lut[j] = Lut[k];
    Lut[k] = ch;
  }
}


//-----------------------------------------------------------------------------
// Konvertiert 8 Bit Zeichenfeld in 6 Bit Codefeld
// nBytes: Anzahl der Zeichen im Feld aBytes
// aBytes: Zeiger auf 8 Bit Zeichenfeld (auf 3 Byte aufgerundet)
// aCodes: Zeiger auf 6 Bit Codefeld, Lnge = (nBytes * 4) / 3
// return: Anzahl 6 Bit Codes im Feld aCodes
int Convert8to6Bit(int nBytes, char *aBytes, char *aCodes)
{
  int  nCodes = 0;
  long data;
  
  while (nBytes > 0)
  {
    if (nBytes > 2)
    { 
      // 3 Byte Block -> 4 x 6-Bit Code
      data = ((long)(BYTE)aBytes[2] << 16) | 
             ((long)(BYTE)aBytes[1] <<  8) | 
              (long)(BYTE)aBytes[0];

      *(aCodes++) = (char)data & 63; data >>= 6;
      *(aCodes++) = (char)data & 63; data >>= 6;
      *(aCodes++) = (char)data & 63; data >>= 6;
      *(aCodes++) = (char)data & 63; 
      nBytes  -= 3;
      aBytes  += 3;
      
      nCodes  += 4;
    }
    else if (nBytes > 1)
    { 
      // 2 Byte -> 3 x 6-Bit Code
      data = ((long)(BYTE)aBytes[1] << 8) | 
              (long)(BYTE)aBytes[0];

      *(aCodes++) = (char)data & 63; data >>= 6;
      *(aCodes++) = (char)data & 63; data >>= 6;
      *(aCodes++) = (char)data & 63; 
      nBytes  -= 2;
      aBytes  += 2;
      
      nCodes  += 3;

    } else {

      // 1 Byte -> 2 x 6-Bit Code
      data = (long)(BYTE)aBytes[0];

      *(aCodes++) = (char)data & 63; data >>= 6;
      *(aCodes++) = (char)data & 63; 
      nBytes  -= 1;
      aBytes  += 1;
      
      nCodes  += 2;
    }
  } // while

  return nCodes;
}

//-----------------------------------------------------------------------------
// Konvertiert 6 Bit Codefeld in 8 Bit Zeichenfeld
// nCodes: Anzahl der Codes im Feld aCodes
// aCodes: Zeiger auf 6 Bit Codefeld
// aBytes: Zeiger auf 8 Bit Zeichenfeld
// return: Anzahl Zeichen im Feld aBytes
int Convert6to8Bit(int nCodes, char *aCodes, char *aBytes)
{
  int nBytes = 0;
  long data;

  while (nCodes > 0)
  {
    if (nCodes > 3)
    { 
      // 4 x 6-Bit Code -> 3 Byte
      data = ((long)(BYTE)aCodes[3] << 18) | 
             ((long)(BYTE)aCodes[2] << 12) | 
             ((long)(BYTE)aCodes[1] <<  6) | 
              (long)(BYTE)aCodes[0];
      *(aBytes++) =  data        & 0xFF;
      *(aBytes++) = (data >>  8) & 0xFF;
      *(aBytes++) = (data >> 16) & 0xFF;

      nBytes  += 3;
      nCodes  -= 4;
      aCodes  += 4;

    }
    else if (nCodes > 2)
    { 
      // 3 x 6-Bit Code -> 2 Byte
      data = ((long)(BYTE)aCodes[2] << 12) | 
             ((long)(BYTE)aCodes[1] <<  6) | 
              (long)(BYTE)aCodes[0];
      *(aBytes++) =  data        & 0xFF;
      *(aBytes++) = (data >>  8) & 0xFF;

      nBytes  += 2;
      nCodes  -= 3;
      aCodes  += 3;

    } else
    {
      // 2 x 6-Bit Code -> 1 Byte
      data = ((long)(BYTE)aCodes[1] <<  6) | 
              (long)(BYTE)aCodes[0];
      *(aBytes++) =  data & 0xFF;

      nBytes  += 1;
      aCodes  += nCodes;
      nCodes   = 0;

    }
  } // while

  return nBytes;
}


//-----------------------------------------------------------------------------
// Konvertiert Codes in ASCII-Form
// Lut: Zeiger auf Look-Up-Tabelle (6-Bit Codes -> ASCII)
// nCodes: Anzahl der Codes im Feld aCode
// aCode:  Zeiger auf Codefeld
// szOut:  Zeiger auf ASCII-Feld (Ausgabe, ohne Null-Abschluss!!!)
void Convert6toASCII(char Lut[64], int nCodes, char *aCode, char *szOut)
{
  int i;

  for (i = 0; i < nCodes; i++) 
  {
    szOut[i] = Lut[ (BYTE) aCode[i] ];

  } // for i;

}


//-----------------------------------------------------------------------------
// Konvertiert ASCII-Codes in 6-Bit-Codes um
// InvLut:   Zeiger auf inverse Look-Up-Tabelle (ASCII -> 6-Bit Codes)
// nAscii:   Anzahl Zeichen in szAscii
// szAscii:  Zeiger auf ASCII-String (ohne '\0' - Abschluss!)
// aCodeOut: Zeiger auf Codefeld (Ausgabe)
// return: Anzahl der Codes im Feld aCodeOut
int ConvertASCIIto6(char InvLut[256], int nAscii, char *szAscii, char *aCodeOut)
{
  int  i, nCodes;
  char ch;

  for (i = nCodes = 0; i < nAscii; i++)
  {
    ch = InvLut[ (BYTE)*(szAscii++) ];
    if (ch != -1) 
      aCodeOut[nCodes++] = ch;

  } // for

  return nCodes;
}

//-----------------------------------------------------------------------------

static long LutRnd = 2139453876;
static int  bInitLut = 0;
static char chLut6Bit[64];
static char chLut8Bit[256];

static void InitStaticLut(long *rnd0)
{
  Init6BitLut(chLut6Bit);
  //Mix6BitLut(rnd0, 256, chLut6Bit);
  Invers6BitLut(chLut6Bit, chLut8Bit);
}

char *GetStatic6BitLut(void)
{
  if (!bInitLut)
  { 
    long rnd = LutRnd;

    bInitLut = 1;
    InitStaticLut(&rnd);
  }
  return chLut6Bit;
}

char *GetStaticInvLut(void)
{
  if (!bInitLut)
  { 
    long rnd = LutRnd;

    bInitLut = 1;
    InitStaticLut(&rnd);
  }
  return chLut8Bit;
}

//-----------------------------------------------------------------------------
// Ldt eine Text-Datei
// szFileName: Zeiger auf Dateiname
// pSize:      Zeiger auf Dateilnge in Bytes (Rckgabewert)
// return:     Zeiger auf Textbuffer (FREE() ausfhren, um freizugeben),
//             NULL bei Fehler.
char * LoadText(char *szFileName, long *pSize)
{
  FILE *fp;
  long  fsize, l;
  int   nRead, nr;
  char *pBuffer = NULL;
    
  *pSize = 0;
  fp = fopen(szFileName, "rb"); // unbedingt binr laden, wegen CR/LF!
  if (fp != NULL)
  {
    // Dateilnge ermitteln
    fseek(fp, 0, SEEK_END);
    fsize = ftell(fp);
    *pSize = fsize;
    fseek(fp, 0, SEEK_SET);
    
    // Speicher reservieren
    pBuffer = MALLOC(fsize + 1); // +1 fr Abschluss-NULL
    if (pBuffer != NULL)
    {
      memset(pBuffer, 0, fsize + 1);
  
      // Text seitenweise laden
      l = 0;
      while (!feof(fp))
      {
        // Fenster lesen
        nr = min(fsize - l, FILE_PAGESIZE);
        nRead  = fread(&pBuffer[l], 1, nr, fp);  // Test OK
        if (nRead == 0) 
          break; // keine Zeichen gelesen
        l += nRead;
  
      } // while (!bEof)
      pBuffer[l] = '\0';

    } // if

    fclose(fp);
  
  } // if

  return pBuffer;
}


//-----------------------------------------------------------------------------
// Speichert eine Text-Datei
// szFileName: Zeiger auf Dateiname
// nText:      Anzahl Zeichen im Buffer pText
// pText:      Zeiger auf Text-Buffer
// return:     TRUE, wenn erfolgreich, sonst FALSE
BOOL SaveText(char *szFileName, long nText, char *pText)
{
  BOOL bSuccess = FALSE;
  FILE *fp;
  long  l, n, nWrite;
    
  fp = fopen(szFileName, "w+b"); // unbedingt binr laden, wegen CR/LF!
  if (fp != NULL)
  {
    // Text seitenweise speichern
    for (l=0; l < nText; /* none */)
    {
      n = min(FILE_PAGESIZE, nText - l);
      
      // Fenster speichern
      nWrite = fwrite(&pText[l], 1, n, fp);
      if (nWrite != n) 
        break; // break for
        
      l += nWrite;

    } // for l

    bSuccess = (l == nText);

    fclose(fp);
  
  } // if

  return bSuccess;
}

//-----------------------------------------------------------------------------
