/*
//-----------------------------------------------------------------------------
OSS - OneTimePad V1.0 (C) 2003 by A. Ogut. All rights reserved. 

License to copy and use this software is granted provided that it 
is identified as the "OSS - OneTimePad" in all material mentioning 
or referencing this software or this function. 

License is also granted to make and use derivative works provided 
that such works are identified as "derived from the OSS - OneTimePad" 
in all material mentioning or referencing the derived work. 

OSS - Ogut Software Systems makes no representations concerning either 
the merchantability of this software or the suitability of this 
software for any particular purpose. It is provided "as is" 
without express or implied warranty of any kind. 

These notices must be retained in any copies of any part of this 
documentation and/or software. 
//-----------------------------------------------------------------------------
*/

// DlgAbort.c: Abort Dialog Tools.

#include <windows.h>
#include <windowsx.h>

#include "DlgAbort.h"


//-----------------------------------------------------------------------------------

BOOL CALLBACK AbortDlg(hDlg, message, wParam, lParam)
HWND hDlg;                                /* window handle of the dialog box */
unsigned message;                         /* type of message                 */
WORD wParam;                              /* message-specific information    */
LONG lParam;
{
  HMENU hMenu;
  USERABORT abr;

  abr = (USERABORT)GetWindowLong(hDlg, DWL_USER);
  if (abr == NULL) 
    return(FALSE);

  switch (message) 
  {
	  case WM_INITDIALOG:		   /* message: initialize dialog box */
        SetDlgItemText(hDlg, IDST_ABORTINFO, abr->AbortText);
        SetWindowText(hDlg, abr->AbortTitel);
        hMenu = GetSystemMenu(hDlg, 0);
        AppendMenu(hMenu, MF_SEPARATOR, 0, NULL);
        AppendMenu(hMenu, MF_STRING, IDB_ABORT, (LPSTR)"&Abbruch");
        break;

    case WM_COMMAND:              /* message: received a command */
        abr->bUserAbort = TRUE;
        break;

    case WM_SIZE:
        AbortAutoPlace(hDlg);
        break;

    case WM_SYSCOMMAND:
       switch (wParam)
       { 
         case IDB_ABORT:
           abr->bUserAbort = TRUE;
           break;
       }
       return(FALSE);
       break;

    default: 
      return (FALSE);               /* Didn't process a message    */
  } // switch

  return(TRUE);
} 


//-----------------------------------------------------------------------------------

HWND WINAPI AbortInstall(HWND hWnd, BOOL Disable, LPSTR  Titel, LPSTR  Text)
{
  HINSTANCE hInst;
  USERABORT abr;

  abr = (USERABORT)GlobalAllocPtr(GPTR, sizeof(USERABORTREC));
  if (abr == NULL) 
    return(NULL);

  abr->AbortTitel   = Titel;
  abr->AbortText    = Text;
  abr->hPWnd        = hWnd;
  abr->bDisable     = Disable;
  if (Disable) EnableWindow(abr->hPWnd, FALSE);
  hInst = (HINSTANCE)GetWindowInstance(hWnd);
  abr->lpAbortProc  = MakeProcInstance((FARPROC)AbortDlg, hInst);
  if (abr->lpAbortProc == NULL) 
    return(FALSE);

  abr->hAbortDlg    = CreateDialog(hInst, MAKEINTRESOURCE(IDD_ABORTDLG), hWnd, (DLGPROC)abr->lpAbortProc);
  if (abr->hAbortDlg == NULL) 
  {
    FreeProcInstance(abr->lpAbortProc);
    return(FALSE);
  }
  AbortAutoPlace(abr->hAbortDlg);
  SetWindowText(abr->hAbortDlg, Titel);
  SetFocus(abr->hAbortDlg);
  abr->bUserAbort = FALSE;
  SetWindowLong(abr->hAbortDlg, DWL_USER, (long)abr);
  
  return(abr->hAbortDlg);
}

//-----------------------------------------------------------------------------------

BOOL WINAPI AbortRemove(HWND hWnd)
{
  USERABORT abr;

  abr = (USERABORT)GetWindowLong(hWnd, DWL_USER);
  if (abr == NULL) 
    return(FALSE);

  if (abr->bDisable) EnableWindow(abr->hPWnd, TRUE);
  DestroyWindow(abr->hAbortDlg);
  FreeProcInstance(abr->lpAbortProc);
  GlobalFreePtr(abr);

  return(TRUE);
}

//-----------------------------------------------------------------------------------

BOOL WINAPI Abort(HWND hWnd)
{
  MSG msg;
  USERABORT abr;

  abr = (USERABORT)GetWindowLong(hWnd, DWL_USER);
  if (abr == NULL) 
    return(FALSE);

  while (!abr->bUserAbort && PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) 
  {
    if (!abr->hAbortDlg || !IsDialogMessage(abr->hAbortDlg, &msg)) 
    {
      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

  } // while
  return(abr->bUserAbort);
}

//-----------------------------------------------------------------------------------

void WINAPI AbortSetText(HWND hWnd, LPSTR Text)
{
  SetDlgItemText(hWnd, IDST_ABORTINFO, Text);
}

//-----------------------------------------------------------------------------------

void WINAPI AbortSetTitle(HWND hWnd, LPSTR Title)
{
  SetWindowText(hWnd, Title);
}

//-----------------------------------------------------------------------------------

void WINAPI AbortDlgSetSize(HWND hWnd, int cx, int cy)
{
  SetWindowPos(hWnd, NULL, 0, 0, cx, cy, SWP_NOMOVE | SWP_NOZORDER);
}

//-----------------------------------------------------------------------------------
void WINAPI AbortAutoPlace(HWND hDlg)
{ 
  RECT rc;
  int b,b2,h,h2;
  HWND hwnd;

  GetClientRect(hDlg, (LPRECT)&rc);
  b=rc.right-rc.left;
  h=rc.bottom-rc.top;
  hwnd=GetDlgItem(hDlg, IDB_ABORT);
  GetClientRect(hwnd, (LPRECT)&rc);
  b2=rc.right-rc.left;
  h2=rc.bottom-rc.top;
  SetWindowPos(hwnd, NULL, (b-b2)/2, h-h2-5, 0, 0, SWP_NOSIZE|SWP_NOZORDER);

  hwnd = GetDlgItem(hDlg, IDST_ABORTINFO);
  SetWindowPos(hwnd, NULL, 5, 5, (b-10), h-h2-10, SWP_NOZORDER);
}

//-----------------------------------------------------------------------------------
