/*
//-----------------------------------------------------------------------------
OSS - OneTimePad V1.0 (C) 2003 by A. Ogut. All rights reserved. 

License to copy and use this software is granted provided that it 
is identified as the "OSS - OneTimePad" in all material mentioning 
or referencing this software or this function. 

License is also granted to make and use derivative works provided 
that such works are identified as "derived from the OSS - OneTimePad" 
in all material mentioning or referencing the derived work. 

OSS - Ogut Software Systems makes no representations concerning either 
the merchantability of this software or the suitability of this 
software for any particular purpose. It is provided "as is" 
without express or implied warranty of any kind. 

These notices must be retained in any copies of any part of this 
documentation and/or software. 
//-----------------------------------------------------------------------------
*/

/* LongMath.c : long Arithmetik Funktionen */

#include <windows.h>
#include <stdlib.h>
#include <math.h>

#include "Longmath.h"


// Multiplikant und Addierwert fr Zufallszahlengenerator
static long _RND_Mul = LONGRND_MUL;
static long _RND_Add = LONGRND_ADD;


//-----------------------------------------------------------------------------------
// Zufallszahlengenerator initialisieren
void WINAPI LongRndInit(long lMul, long lAdd)
{
  _RND_Mul = lMul;
  _RND_Add = lAdd;
}


//-----------------------------------------------------------------------------------
// Zufallszahl ermitteln 
long WINAPI LongRnd(long far *rndval)
{
  long lastrnd;

  lastrnd = *rndval;
  lastrnd = ((_RND_Mul * lastrnd) + _RND_Add) & 0x7FFFFFFFL;
  *rndval = lastrnd;
  return lastrnd;
}


//-----------------------------------------------------------------------------------
// Zufallszahl im Intervall [0..max[ erzeugen
long WINAPI LongRandom(long far *rndval, long max)
{
  long lastrnd;

  lastrnd = *rndval;
  lastrnd = ((_RND_Mul * lastrnd) + _RND_Add) & 0x7FFFFFFFL;
  *rndval = lastrnd;
  lastrnd = MulDiv(lastrnd, max, LONGRND_MAXRAND);
  
  // statisch gleich verteilt im Bereich [0..max[
  if (lastrnd >= max) lastrnd = 0; 

  return lastrnd;
}


//-----------------------------------------------------------------------------------
// Prft, ob die vorgegebene Zahl eine Primzahl ist
// p:      zu prfende Zahl
// sqrtp:  Wurzel der Zahl p ( sqrtp = (long)sqrt(p) )
// return: TRUE wenn die Zahl eine Primzahl ist
BOOL WINAPI IsPrime(long p, long sqrtp)
{
  if (p == 2) return TRUE; // 2 ist Primzahl 
  if ((p & 1) == 0) return FALSE; // Gerade Zahlen sind keine Primzahlen
  if (p == 0 || p == 1) return FALSE; // 0 und 1 sind keine Primzahlen
  if (p == 3) return TRUE; // 3 ist Primzahl 
  
  while ((long)sqrtp > 1) {
    if ((p % sqrtp) == 0) return FALSE; // keine Primzahl
    if ((sqrtp & 1) == 0) sqrtp -= 1;
    else                  sqrtp -= 2;
  }
  return TRUE; // Primzahl gefunden
}


//-----------------------------------------------------------------------------------
// Ermittelt die nchste grere Primzahl > p
long WINAPI NextPrime(long p)
{
  p++;
  while (!IsPrime(p, (long)sqrt( (double)p ) ) ) p++;
  return p;
}


//-----------------------------------------------------------------------------------
// Ermittelt die nchste kleinere Primzahl < p
long WINAPI PrevPrime(long p)
{
  p--;
  while (!IsPrime(p, (long)sqrt( (double)p ) ) ) p--;
  return p;
}

//-----------------------------------------------------------------------------------

