/*
//-----------------------------------------------------------------------------
OSS - OneTimePad V1.0 (C) 2003 by A. Ogut. All rights reserved. 

License to copy and use this software is granted provided that it 
is identified as the "OSS - OneTimePad" in all material mentioning 
or referencing this software or this function. 

License is also granted to make and use derivative works provided 
that such works are identified as "derived from the OSS - OneTimePad" 
in all material mentioning or referencing the derived work. 

OSS - Ogut Software Systems makes no representations concerning either 
the merchantability of this software or the suitability of this 
software for any particular purpose. It is provided "as is" 
without express or implied warranty of any kind. 

These notices must be retained in any copies of any part of this 
documentation and/or software. 
//-----------------------------------------------------------------------------
*/

/* Memalloc.c : Memory allocation debug utilities */

#include <windows.h>
#include <malloc.h> //_msize()
#include <stdio.h>
#include <tchar.h>

#include "Preproc.h"
#include "Memalloc.h"


//-----------------------------------------------------------------------------------

#ifdef MALLOC_DEBUG

static LONG malloc_max   = 0, // maximul memory allocation in bytes
            malloc_use   = 0, // current memory allocation in bytes
            malloc_count = 0; // number of current memory allocations

//-----------------------------------------------------------------------------------
// Debugout a value
static void dbg_val(LPTSTR szFmt, DWORD val)
{
  TCHAR szInfo[128];

  _stprintf(szInfo, szFmt, val);
  DEBUGOUT(szInfo);
}

//-----------------------------------------------------------------------------------
// MALLOC()-wrapper for debugging
void * WINAPI dbg_malloc(LONG s)
{
  void *ptr;
  LONG  u,m,c;

  ptr = malloc(s);
  if (ptr != NULL) {
    
    malloc_use += s;
    malloc_count++;

    if (malloc_use > malloc_max) 
      malloc_max = malloc_use; 
    
    u = malloc_use;
    m = malloc_max;
    c = malloc_count;

#ifdef MALLOC_FULLDEBUGINFO    

    dbg_val(TEXT("malloc:\tptr = 0x%08lx\t"), (LONG)ptr);
    dbg_val(TEXT("size = %8ld\t"), s);

#else
    
    DEBUGOUT(TEXT("malloc:\t"));

#endif

    dbg_val(TEXT("cntr = %8ld\t"), c);
    dbg_val(TEXT("used = %8ld\t"), u);
    dbg_val(TEXT("max = %8ld\n"),  m);

  }
  return ptr;
}

//-----------------------------------------------------------------------------------
// FREE()-wrapper for debugging
void WINAPI dbg_free(void *ptr)
{
  LONG s, u,m,c;

  if (ptr != NULL) {
    
    s = (LONG)_msize(ptr);
    malloc_use -= s;
    malloc_count--;

    u = malloc_use;
    m = malloc_max;
    c = malloc_count;

#ifdef MALLOC_FULLDEBUGINFO    

    dbg_val(TEXT("free:\tptr = 0x%08lx\t"), (LONG)ptr);
    dbg_val(TEXT("size = %8ld\t"), s);

#else

    DEBUGOUT(TEXT("free:\t"));

#endif

    dbg_val(TEXT("cntr = %8ld\t"), c);
    dbg_val(TEXT("used = %8ld\t"), u);
    dbg_val(TEXT("max = %8ld\n"),  m);

    free(ptr);
  }
}

//-----------------------------------------------------------------------------------

#endif
