/*
//-----------------------------------------------------------------------------
OSS - OneTimePad V1.0 (C) 2003 by A. Ogut. All rights reserved. 

License to copy and use this software is granted provided that it 
is identified as the "OSS - OneTimePad" in all material mentioning 
or referencing this software or this function. 

License is also granted to make and use derivative works provided 
that such works are identified as "derived from the OSS - OneTimePad" 
in all material mentioning or referencing the derived work. 

OSS - Ogut Software Systems makes no representations concerning either 
the merchantability of this software or the suitability of this 
software for any particular purpose. It is provided "as is" 
without express or implied warranty of any kind. 

These notices must be retained in any copies of any part of this 
documentation and/or software. 
//-----------------------------------------------------------------------------
*/

// OneTimePad.cpp : Legt das Klassenverhalten fr die Anwendung fest.
//

#include "stdafx.h"
#include <io.h>
#include "OneTimePad.h"
#include "OneTimePadDlg.h"
#include "DlgLicenseWiz1.h"
#include "DlgLicenseWiz2.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//
// Globale Variablen
//
char szAppIniFile[_MAX_PATH] = "";   // Programmparameterdateiname
char szKeyFile[_MAX_PATH] = "";      // Schlsseldateiname

APPSETTINGS AppSettings;       // Anwendungseinstellungen


//
// Konstanten fr Programmparameterdatei (*.INI)
//
const char szAppIniSection[] = "OneTimePad"; // Abschnittsname der Ini-Datei
const char szAppIniSectionKeyHistory[] = "Key-History"; // Abschnittsname der Ini-Datei

const char szIniAppRndSeed[]     = "RndSeed";
const char szIniAppRndNum[]      = "RndNum";
const char szIniAppRndMul[]      = "RndMul";
const char szIniAppRndAdd[]      = "RndAdd";

const char szIniAppMixNumWindow[]= "MixWindowSize";
const char szIniAppMixNumExchg[] = "MixNumExchg";
const char szIniAppMixNumPass[]  = "MixNumPass";

const char szIniAppRepLimWindow[]= "RepLimWindowSize";
const char szIniAppDelafterOperation[]= "DeleteafterOperation";
const char szIniAppConsumeKey[]=   "ConsumeKey";

const char szAppIniDecodePrefix[]= "DecodePrefix";
const char szAppIniCipherSuffix[]= "CipherSuffix";
const char szAppIniAsciiSuffix[] = "AsciiSuffix";
//const char szAppIniViewMode[]    = "ViewMode";
const char szAppIniIncSubDirs[]  = "IncludeSubDirs";
const char szAppIniDropFMask[]   = "DragDropFileMask";
const char szAppIniWindowPos[]   = "WindowPos";
const char szAppIniLicense[]     = "LicenseAgreement";


const char szIniKeyHistoryIndex[]= "Index";
const char szIniKeyHistoryName[] = "FName";


const char szKeyFileMask[]       = "*.pad";     // Filemaske fr Schlsseldateinamen
const char szKeyFileMask2[]      = "Schssel Dateien (*.pad)|*.pad|Alle Dateien (*.*)|*.*|";

const char szAllFileMask[]       = "*.*";
const char szAllFileMask2[]      = "Alle Dateien (*.*)|*.*|";

const char szTxtFileMask[]       = "*.txt";
const char szTxtFileMask2[]      = "Text Dateien (*.txt)|*.txt|";


/////////////////////////////////////////////////////////////////////////////

char * ParseName(char *szName)
{
  char *ch, *c;

  ch = szName;
  
  while (*ch == ' ') ch++;
  
  if (*ch == '"')
  {
    c = strchr(++ch, '"');
    if (c != NULL) *c = '\0';
  }
  return ch;
}

//-----------------------------------------------------------------------------
char *FormatLongToStr(long l, char *chTmp)
{
  if (l < 1024) 
  { 
    // Byte
    sprintf(chTmp, "%ld", l);

  } else if (l < (DWORD)1024*1024) 
  { 
    // KB
    sprintf(chTmp, "%4.1f KB", (double)l/(1024.0));

  } else
  { 
    // MB
    sprintf(chTmp, "%4.1f MB", (double)l/(1024.0*1024.0));
  } 

  return chTmp;
}

//-----------------------------------------------------------------------------
// Ermittelt, ob der angegebene Dateiname ein Verzeichnis ist
// szFile: Zeiger auf Dateiname
// return: TRUE, wenn Verzeichnis, sonst FALSE bei Dateien
BOOL IsDirectory(char *szFile)
{
  BOOL bRet = FALSE;
  struct _finddata_t fi; 
  long fh;
  
  memset(&fi, 0, sizeof(fi));
  fi.attrib = _A_NORMAL;

  fh = _findfirst(szFile, &fi);
  if (fh != -1)
  {
    bRet = (fi.attrib & _A_SUBDIR) != 0;
    _findclose(fh);
  }
  return bRet;
}

//-----------------------------------------------------------------------------
// Fllt eine Listbox mit Dateinamen (Unterverzeichnisse werden einbezogen)
// pListbox: Zeiger auf Listbox-Objekt
// szPrefix: Zeiger auf Prefix-Text
// szFilespec: Zeiger auf Dateiname/Verzeichnisname
// return:   Anzahl Elemente hinzugefgt.
int ListboxFillFiles(BOOL bDragDlg, CListBox *pListbox, char *szPrefix, char *szFilespec, APPSETTINGS *pSettings, CWnd *pWnd)
{
  char szTmp[_MAX_PATH];
  struct _finddata_t fi; 
  long fh;
  int  n = 0;

  if (!IsDirectory(szFilespec))
  {
    sprintf(szTmp, "%s%s (%ld)", szPrefix, szFilespec, FileSize(szFilespec, NULL));
    pListbox->AddString(szTmp);
    return 1;
  }
  
  //
  // Drag & Drop Dialogbox anzeigen bei Bedarf
  //
  if (bDragDlg)
  {
    CDlgDragDrop dlgDragDrop(pSettings->szDragDropMask, sizeof(pSettings->szDragDropMask), &pSettings->bIncSubDirs, pWnd);

    pWnd->SetForegroundWindow();
    pWnd->BringWindowToTop();

    if (dlgDragDrop.DoModal() != IDOK)
      return 0;

  }
  
  //
  // Dateien durchsuchen und bearbeiten
  //
  strcpy(szTmp, szFilespec);
  strcat(szTmp, "\\");
  strcat(szTmp, pSettings->szDragDropMask);

  memset(&fi, 0, sizeof(fi));
  fi.attrib = _A_NORMAL;

  fh = _findfirst(szTmp, &fi);
  if (fh != -1)
  {
    do {
      // Name verarbeiten
      if ((fi.attrib & _A_SUBDIR) == 0 )
      {
        sprintf(szTmp, "%s%s\\%s (%ld)", szPrefix, szFilespec, fi.name, fi.size);
        pListbox->AddString(szTmp);
        n++;
      }

    } while ( _findnext(fh, &fi) == 0);

    _findclose(fh);
  }

  //
  // Unterverzeichnisse suchen und bearbeiten
  //
  if (pSettings->bIncSubDirs)
  {
    strcpy(szTmp, szFilespec);
    strcat(szTmp, "\\*.*");

    memset(&fi, 0, sizeof(fi));
    fi.attrib = _A_NORMAL;

    fh = _findfirst(szTmp, &fi);
    if (fh != -1)
    {
      do {
        // Name verarbeiten...
        if ((fi.attrib & _A_SUBDIR) != 0 )
        {
          if (stricmp(fi.name, ".") == 0) continue;
          if (stricmp(fi.name, "..") == 0) continue;

          strcpy(szTmp, szFilespec);
          strcat(szTmp, "\\");
          strcat(szTmp, fi.name);
          n += ListboxFillFiles(FALSE, pListbox, szPrefix, szTmp, pSettings, pWnd);
        }

      } while ( _findnext(fh, &fi) == 0);

      _findclose(fh);
    }
  }

  return n;
}


/////////////////////////////////////////////////////////////////////////////
// CMyOneTimePadApp

BEGIN_MESSAGE_MAP(CMyOneTimePadApp, CWinApp)
	//{{AFX_MSG_MAP(CMyOneTimePadApp)
		// HINWEIS - Hier werden Mapping-Makros vom Klassen-Assistenten eingefgt und entfernt.
		//    Innerhalb dieser generierten Quelltextabschnitte NICHTS VERNDERN!
	//}}AFX_MSG
	ON_COMMAND(ID_HELP, CWinApp::OnHelp)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMyOneTimePadApp Konstruktion

CMyOneTimePadApp::CMyOneTimePadApp()
{
	// ZU ERLEDIGEN: Hier Code zur Konstruktion einfgen
	// Alle wichtigen Initialisierungen in InitInstance platzieren
}

/////////////////////////////////////////////////////////////////////////////
// Das einzige CMyOneTimePadApp-Objekt

CMyOneTimePadApp theApp;

/////////////////////////////////////////////////////////////////////////////
// CMyOneTimePadApp Initialisierung

BOOL CMyOneTimePadApp::InitInstance()
{
  AfxEnableControlContainer();
  
	// Standardinitialisierung
	// Wenn Sie diese Funktionen nicht nutzen und die Gre Ihrer fertigen 
	//  ausfhrbaren Datei reduzieren wollen, sollten Sie die nachfolgenden
	//  spezifischen Initialisierungsroutinen, die Sie nicht bentigen, entfernen.

#ifdef _AFXDLL
	Enable3dControls();			// Diese Funktion bei Verwendung von MFC in gemeinsam genutzten DLLs aufrufen
#else
	Enable3dControlsStatic();	// Diese Funktion bei statischen MFC-Anbindungen aufrufen
#endif

  //
  // Einstellungen initialisieren
  //
  memset(&AppSettings, 0, sizeof(APPSETTINGS));

  // Programmparameter aus der INI-Datei einlesen
  SetupIniFile();
  LoadSettings();

  if (!AppSettings.bLicenseAgreement)
    AppSettings.bLicenseAgreement = AskLicenseAgreement();

  if (!AppSettings.bLicenseAgreement)
  {
    return FALSE;
  }

  //
	// Hauptdialog der Anwendung ffnen und starten
  //
  CMyOneTimePadDlg dlg;
	m_pMainWnd = &dlg;
	int nResponse = dlg.DoModal();
	if (nResponse == IDOK)
	{
		// ZU ERLEDIGEN: Fgen Sie hier Code ein, um ein Schlieen des
		//  Dialogfelds ber OK zu steuern
	}
	else if (nResponse == IDCANCEL)
	{
		// ZU ERLEDIGEN: Fgen Sie hier Code ein, um ein Schlieen des
		//  Dialogfelds ber "Abbrechen" zu steuern
	}

  // Einstellungen zurckspeichern
  SaveSettings();

	// Da das Dialogfeld geschlossen wurde, FALSE zurckliefern, so dass wir die
	//  Anwendung verlassen, anstatt das Nachrichtensystem der Anwendung zu starten.
	return FALSE;
}


//-----------------------------------------------------------------------------
// Ini-Datei der Anwendung ermitteln.
void CMyOneTimePadApp::SetupIniFile()
{
  // Parameterdatei ermitteln
  _getcwd(szAppIniFile, sizeof(szAppIniFile));
  strcat(szAppIniFile, "\\");
  strcat(szAppIniFile, m_pszProfileName);

  //First free the string allocated by MFC at CWinApp startup.
  //The string is allocated before InitInstance is called.
  free((void*)m_pszProfileName);
  //Change the name of the .INI file.
  //The CWinApp destructor will free the memory.
  m_pszProfileName=_tcsdup(_T(szAppIniFile));
}


//-----------------------------------------------------------------------------
// Ldt die Einstellungen des Programms von der Ini-Datei.
void CMyOneTimePadApp::LoadSettings()
{
  char szBuf[256];
  char szItem[128];
  int i;

  AppSettings.bLicenseAgreement = GetPrivateProfileInt(szAppIniSection, szAppIniLicense, 0, szAppIniFile); 

  AppSettings.lRndSeed      = GetPrivateProfileInt(szAppIniSection, szIniAppRndSeed,      7777777,     szAppIniFile);
  AppSettings.lRndNum       = GetPrivateProfileInt(szAppIniSection, szIniAppRndNum,       1024*1024,   szAppIniFile);
  AppSettings.lRndMul       = GetPrivateProfileInt(szAppIniSection, szIniAppRndMul,       LONGRND_MUL, szAppIniFile);
  AppSettings.lRndAdd       = GetPrivateProfileInt(szAppIniSection, szIniAppRndAdd,       LONGRND_ADD, szAppIniFile);
  LongRndInit(AppSettings.lRndMul, AppSettings.lRndAdd);

  AppSettings.lMixNumWindow = GetPrivateProfileInt(szAppIniSection, szIniAppMixNumWindow, 8192,  szAppIniFile);
  AppSettings.lMixNumExchg  = GetPrivateProfileInt(szAppIniSection, szIniAppMixNumExchg,  16384, szAppIniFile);
  AppSettings.lMixNumPass   = GetPrivateProfileInt(szAppIniSection, szIniAppMixNumPass,   4,     szAppIniFile);
  AppSettings.lRepLimWindow = GetPrivateProfileInt(szAppIniSection, szIniAppRepLimWindow, 64,    szAppIniFile);

  // Dialogbox-Einstellungen laden
  AppSettings.bDeleteafterOperation = GetPrivateProfileInt(szAppIniSection, szIniAppDelafterOperation, 0, szAppIniFile); 
  AppSettings.bConsumeKey           = GetPrivateProfileInt(szAppIniSection, szIniAppConsumeKey,    0, szAppIniFile); 
  //AppSettings.nViewMode             = GetPrivateProfileInt(szAppIniSection, szAppIniViewMode,      0, szAppIniFile); 
  AppSettings.bIncSubDirs           = GetPrivateProfileInt(szAppIniSection, szAppIniIncSubDirs,    1, szAppIniFile); 

  GetPrivateProfileString(szAppIniSection, szAppIniDecodePrefix, DEFAULT_DECODEPREFIX, AppSettings.szDecodePrefix, sizeof(AppSettings.szDecodePrefix), szAppIniFile);
  GetPrivateProfileString(szAppIniSection, szAppIniCipherSuffix, DEFAULT_CIPHERSUFFIX, AppSettings.szCipherSuffix, sizeof(AppSettings.szCipherSuffix), szAppIniFile);
  GetPrivateProfileString(szAppIniSection, szAppIniAsciiSuffix,  DEFAULT_ASCIISUFFIX,  AppSettings.szAsciiSuffix,  sizeof(AppSettings.szAsciiSuffix),  szAppIniFile);
  GetPrivateProfileString(szAppIniSection, szAppIniDropFMask,    DEFAULT_DROPFMASK,    AppSettings.szDragDropMask, sizeof(AppSettings.szDragDropMask), szAppIniFile);

  for (i = AppSettings.nKeyFiles = 0; i < MAX_KEYSTREAM_HISTORY; i++)
  {
    sprintf(szItem, "%s%d", szIniKeyHistoryIndex, i+1);
    AppSettings.KeyFileHistory[AppSettings.nKeyFiles].nIndex = GetPrivateProfileInt(szAppIniSectionKeyHistory, szItem, 0, szAppIniFile);

    sprintf(szItem, "%s%d", szIniKeyHistoryName, i+1);
    GetPrivateProfileString(szAppIniSectionKeyHistory, szItem, "", szBuf, sizeof(szBuf), szAppIniFile);
    strcpy(AppSettings.KeyFileHistory[AppSettings.nKeyFiles].szKeyFile, ParseName(szBuf));
    
    if (strlen(AppSettings.KeyFileHistory[AppSettings.nKeyFiles].szKeyFile) != '\0')
      AppSettings.nKeyFiles++;

  } // for i

}


//-----------------------------------------------------------------------------
// Speichert die Einstellungen des Programms in die Ini-Datei ab.
void CMyOneTimePadApp::SaveSettings()
{
  int  i;
  char szBuf[256];
  char szItem[128];

  sprintf(szBuf, "%lu", AppSettings.bLicenseAgreement);
  WritePrivateProfileString(szAppIniSection, szAppIniLicense, szBuf, szAppIniFile);

  sprintf(szBuf, "%lu", AppSettings.lRndSeed);
  WritePrivateProfileString(szAppIniSection, szIniAppRndSeed, szBuf, szAppIniFile);

  sprintf(szBuf, "%lu", AppSettings.lRndNum);
  WritePrivateProfileString(szAppIniSection, szIniAppRndNum, szBuf, szAppIniFile);

  sprintf(szBuf, "%lu", AppSettings.lRndMul);
  WritePrivateProfileString(szAppIniSection, szIniAppRndMul, szBuf, szAppIniFile);

  sprintf(szBuf, "%lu", AppSettings.lRndAdd);
  WritePrivateProfileString(szAppIniSection, szIniAppRndAdd, szBuf, szAppIniFile);

  sprintf(szBuf, "%lu", AppSettings.lMixNumWindow);
  WritePrivateProfileString(szAppIniSection, szIniAppMixNumWindow, szBuf, szAppIniFile);

  sprintf(szBuf, "%lu", AppSettings.lMixNumExchg);
  WritePrivateProfileString(szAppIniSection, szIniAppMixNumExchg,  szBuf, szAppIniFile);

  sprintf(szBuf, "%lu", AppSettings.lMixNumPass);
  WritePrivateProfileString(szAppIniSection, szIniAppMixNumPass,   szBuf, szAppIniFile);

  sprintf(szBuf, "%lu", AppSettings.lRepLimWindow);
  WritePrivateProfileString(szAppIniSection, szIniAppRepLimWindow, szBuf, szAppIniFile);


  // Dialogbox-Einstellungen speichern
  sprintf(szBuf, "%lu", AppSettings.bDeleteafterOperation);
  WritePrivateProfileString(szAppIniSection, szIniAppDelafterOperation, szBuf, szAppIniFile); 

  sprintf(szBuf, "%lu", AppSettings.bConsumeKey);
  WritePrivateProfileString(szAppIniSection, szIniAppConsumeKey, szBuf, szAppIniFile); 

  WritePrivateProfileString(szAppIniSection, szAppIniDecodePrefix,  AppSettings.szDecodePrefix, szAppIniFile);
  WritePrivateProfileString(szAppIniSection, szAppIniCipherSuffix,  AppSettings.szCipherSuffix, szAppIniFile);
  WritePrivateProfileString(szAppIniSection, szAppIniAsciiSuffix,   AppSettings.szAsciiSuffix,  szAppIniFile);

  //sprintf(szBuf, "%lu", AppSettings.nViewMode);
  //WritePrivateProfileString(szAppIniSection, szAppIniViewMode, szBuf, szAppIniFile); 

  sprintf(szBuf, "%lu", AppSettings.bIncSubDirs);
  WritePrivateProfileString(szAppIniSection, szAppIniIncSubDirs, szBuf, szAppIniFile); 
  WritePrivateProfileString(szAppIniSection, szAppIniDropFMask, AppSettings.szDragDropMask, szAppIniFile); 

  for (i = 0; i < MAX_KEYSTREAM_HISTORY; i++)
  {
    sprintf(szItem, "%s%d", szIniKeyHistoryIndex, i+1);
    if (i < AppSettings.nKeyFiles) 
    {
      sprintf(szBuf, "%lu", AppSettings.KeyFileHistory[i].nIndex);
      WritePrivateProfileString(szAppIniSectionKeyHistory, szItem, szBuf, szAppIniFile);

    } else WritePrivateProfileString(szAppIniSectionKeyHistory, szItem, NULL, szAppIniFile);

    sprintf(szItem, "%s%d", szIniKeyHistoryName, i+1);
    if (i < AppSettings.nKeyFiles) 
    {
      sprintf(szBuf, "\"%s\"", AppSettings.KeyFileHistory[i].szKeyFile);
      WritePrivateProfileString(szAppIniSectionKeyHistory, szItem, szBuf, szAppIniFile);

    } else WritePrivateProfileString(szAppIniSectionKeyHistory, szItem, NULL, szAppIniFile);
  }

}


//-----------------------------------------------------------------------------
// Fgt eine Schlsseldatei in die History ein. Doppelte Eintrge werden
//  verhindert. Aktualisiert die History so, da zuletzt eingetragene
//  Schlsseldatei zum Ver/Entschlsseln benutzt wird.
void CMyOneTimePadApp::AddKeyFiletoHistory(char *szKeyFile)
{
  int           i;
  KEYSTREAMINFO tmp;

  // Eintrag schon vorhanden?
  for (i = 0; i < AppSettings.nKeyFiles; i++)
  {
    if ( strncmp(AppSettings.KeyFileHistory[i].szKeyFile, szKeyFile, strlen(szKeyFile)) == 0 )
    {
      // Eintrag gefunden -> auf Index 0 bringen
      while (i != 0)
      {
        tmp = AppSettings.KeyFileHistory[i];
        AppSettings.KeyFileHistory[i]   = AppSettings.KeyFileHistory[i-1];
        AppSettings.KeyFileHistory[i-1] = tmp;
        i--;
      }
      return;
    }
  
  }
  // kein Eintrag gefunden -> in die Liste hinzufgen

  // Historyliste nach unten durchrutschen
  for (i = MAX_KEYSTREAM_HISTORY-1; i > 0; i--)
  {
    AppSettings.KeyFileHistory[i] = AppSettings.KeyFileHistory[i-1];
  }
  
  // neuen Eintrag an die oberste Stelle setzen
  AppSettings.KeyFileHistory[0].nIndex = 0;
  AppSettings.KeyFileHistory[0].nSize  = FileSize(szKeyFile, &AppSettings.KeyFileHistory[0].bRW);
  strcpy(AppSettings.KeyFileHistory[0].szKeyFile, szKeyFile);
  
  if (AppSettings.nKeyFiles < MAX_KEYSTREAM_HISTORY) 
    AppSettings.nKeyFiles++;

}


//-----------------------------------------------------------------------------
// Festerkoordinaten speichern
void CMyOneTimePadApp::SaveWindowPos(RECT rc)
{
  char buf[128];

  sprintf(buf, "%d %d %d %d", rc.left, rc.top, rc.right, rc.bottom);
  WritePrivateProfileString(szAppIniSection, szAppIniWindowPos, buf,  szAppIniFile);

}


//-----------------------------------------------------------------------------
// Festerkoordinaten laden
BOOL CMyOneTimePadApp::LoadWindowPos(RECT &rc)
{
  char buf[128];

  memset(&rc, 0, sizeof(RECT));
  
  GetPrivateProfileString(szAppIniSection, szAppIniWindowPos, "",  buf, sizeof(buf), szAppIniFile);
  return sscanf(buf, "%d %d %d %d", &rc.left, &rc.top, &rc.right, &rc.bottom) == 4;
}

//-----------------------------------------------------------------------------


CDlgLicenseWizard::CDlgLicenseWizard(UINT nIDCaption = 0, CWnd *pParentWnd = NULL, UINT iSelectPage = 0 )
 : CPropertySheet(nIDCaption, pParentWnd, iSelectPage)
{
  Answer = 0;
}

CDlgLicenseWizard::~CDlgLicenseWizard()
{
}

//-----------------------------------------------------------------------------

BOOL CMyOneTimePadApp::AskLicenseAgreement()
{
  CDlgLicenseWizard dlgLicenseWiz(IDS_APP, NULL, 0);
  CDlgLicenseWiz1 dlgWiz1(&dlgLicenseWiz);
  CDlgLicenseWiz2 dlgWiz2(&dlgLicenseWiz);

  dlgLicenseWiz.AddPage((CPropertyPage *)&dlgWiz1);
  dlgLicenseWiz.AddPage((CPropertyPage *)&dlgWiz2);
      
  dlgLicenseWiz.m_psh.dwFlags  |= PSH_NOAPPLYNOW | PSH_USEICONID | PSH_WIZARD;
  dlgLicenseWiz.m_psh.hInstance = theApp.m_hInstance;
  dlgLicenseWiz.m_psh.pszIcon   = MAKEINTRESOURCE(IDI_KEY);

  UINT ret = dlgLicenseWiz.DoModal();

  // Ergebnis irrelevant
  if (ret == ID_WIZFINISH)
  {
    int Antworten = (1 << 0) |
                    (0 << 1) |
                    (0 << 2) |
                    (0 << 3) |
                    (1 << 4) |
                    (0 << 5);
                     
                    
    if (dlgLicenseWiz.Answer != Antworten)
    {
      MessageBeep(0);
      MessageBox(NULL, CString((LPCSTR)IDS_LICENSEDENY), CString((LPCSTR)IDS_ERROR), MB_OK | MB_ICONSTOP);
      return FALSE;

    } else 
    {
      MessageBox(NULL, CString((LPCSTR)IDS_LICENSEOK), CString((LPCSTR)IDS_APP), MB_OK | MB_ICONINFORMATION);
      return TRUE;
    }
  }
  
  return FALSE;
}

//-----------------------------------------------------------------------------
