/*
//-----------------------------------------------------------------------------
OSS - OneTimePad V1.0 (C) 2003 by A. Ogut. All rights reserved. 

License to copy and use this software is granted provided that it 
is identified as the "OSS - OneTimePad" in all material mentioning 
or referencing this software or this function. 

License is also granted to make and use derivative works provided 
that such works are identified as "derived from the OSS - OneTimePad" 
in all material mentioning or referencing the derived work. 

OSS - Ogut Software Systems makes no representations concerning either 
the merchantability of this software or the suitability of this 
software for any particular purpose. It is provided "as is" 
without express or implied warranty of any kind. 

These notices must be retained in any copies of any part of this 
documentation and/or software. 
//-----------------------------------------------------------------------------
*/

// OneTimePad.h : Haupt-Header-Datei fr die Anwendung OneTap
//

#if !defined(AFX_ONETIMEPAD_H__1E8EB5C2_2A48_11D7_A1AF_909CBD5DBF43__INCLUDED_)
#define AFX_ONETIMEPAD_H__1E8EB5C2_2A48_11D7_A1AF_909CBD5DBF43__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#ifndef __AFXWIN_H__
	#error include 'stdafx.h' before including this file for PCH
#endif

#include "Resource.h"		// Hauptsymbole
#include "OneTimePadTl.h"

/////////////////////////////////////////////////////////////////////////////
// Globals:
//
class CMyOneTimePadApp;
extern CMyOneTimePadApp theApp;

extern const char szKeyFileMask[];   // Filemaske fr Schlssel Dateinamen
extern const char szKeyFileMask2[];

extern const char szAllFileMask[];   // Filemaske fr Alle Dateinamen
extern const char szAllFileMask2[];

extern const char szTxtFileMask[];   // Filemaske fr Text Dateinamen
extern const char szTxtFileMask2[];

//
// Standardwerte fr Dateinamen
//
#define DEFAULT_DECODEPREFIX  "~"
#define DEFAULT_CIPHERSUFFIX  ".1tp"
#define DEFAULT_ASCIISUFFIX   ".asc"
#define DEFAULT_DROPFMASK     "*.*"

//
// Konstanten fr die Anwendung
//
enum { 
  MIN_PWD_SIZE          = 0,     // minimale Passwortlnge
  MAX_PWD_SIZE          = 255,   // maximale Passwortlnge
  MAX_FILEMASK_SIZE     = 32,    // maximale Dateimaskenlnge
  //MAX_TEXT_BUFFER       = 32767, // maximale Textlnge im Text-Fenster
  MAX_KEYSTREAM_HISTORY = 8      // Anzahl der max. Schlsseldateien in der Liste
};


typedef struct { // Programmeinstellungen
  int   bLicenseAgreement;   // Lizenzabkommen behandelt
  //int   nViewMode;           // Anzeige-Modus (Text/Datei)
  
  char  szDragDropMask[MAX_FILEMASK_SIZE]; // Filenamensmaske fr Drag und Drop
  BOOL  bIncSubDirs;         // Unterverzeichnisse einbeziehen

  char  szCipherSuffix[MAX_FILEMASK_SIZE], // Namenserweiterung fr verschlsselte Dateien
        szAsciiSuffix[MAX_FILEMASK_SIZE],  // Namenserweiterung fr ASCII-6 Bit kodierte Dateien
        szDecodePrefix[MAX_FILEMASK_SIZE]; // Namensprefix fr entschlsselte Dateien
  
  BOOL  bDeleteafterOperation, // Lschen nach der Operation - Flag
        bConsumeKey;       // Schlssel verbrauchen und danach lschen

  long  lRndSeed;         // Zufallsvariable
  long  lRndNum;          // Anzahl der Zufallszahlenwerte (fr Schlsselgenerator)
  long  lRndMul;          // Zufallsvariable: Multiplikand
  long  lRndAdd;          // Zufallsvariable: Summand

  long  lMixNumWindow;    // Fenstergrsse zum Mixen der Schlsseldatei
  long  lMixNumExchg;     // Anzahl der Vertauschungen im Fenster
  long  lMixNumPass;      // Anzahl der Mischdurchlufe der Schlsseldatei

  long  lRepLimWindow;    // Fenstergrsse fr Wiederholungseliminator

  int   nKeyFiles;          // Anzahl der Schlssel in der Historyliste
  KEYSTREAMINFO KeyFileHistory[MAX_KEYSTREAM_HISTORY]; // Historyliste der Schlssel

} APPSETTINGS;

extern APPSETTINGS AppSettings;     // Anwendungseinstellungen


char *FormatLongToStr(long l, char *chTmp);
BOOL IsDirectory(char *szFile);
int  ListboxFillFiles(BOOL bDragDlg, CListBox *pListbox, char *szPrefix, char *szFilespec, APPSETTINGS *pSettings, CWnd *pWnd);


/////////////////////////////////////////////////////////////////////////////
// CMyOneTimePadApp:
// Siehe OneTimePad.cpp fr die Implementierung dieser Klasse
//

class CMyOneTimePadApp : public CWinApp
{
public:
	CMyOneTimePadApp();

// berladungen
	// Vom Klassenassistenten generierte berladungen virtueller Funktionen
	//{{AFX_VIRTUAL(CMyOneTimePadApp)
	public:
	virtual BOOL InitInstance();
	//}}AFX_VIRTUAL

// Implementierung
public:
  BOOL AskLicenseAgreement(void);
  void SetupIniFile(void);
  void LoadSettings(void);
  void SaveSettings(void);
  void SaveWindowPos(RECT rc);
  BOOL LoadWindowPos(RECT &rc);

  void AddKeyFiletoHistory(char *szKeyFile);


	//{{AFX_MSG(CMyOneTimePadApp)
		// HINWEIS - An dieser Stelle werden Member-Funktionen vom Klassen-Assistenten eingefgt und entfernt.
		//    Innerhalb dieser generierten Quelltextabschnitte NICHTS VERNDERN!
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};


/////////////////////////////////////////////////////////////////////////////
// CDlgLicenseWizard 

class CDlgLicenseWizard : public CPropertySheet
{

// Konstruktion
public:
	CDlgLicenseWizard(UINT nIDCaption, CWnd *pParentWnd, UINT iSelectPage);
	~CDlgLicenseWizard();
  
  int Answer;

// Implementierung

};


/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ fgt unmittelbar vor der vorhergehenden Zeile zustzliche Deklarationen ein.

#endif // !defined(AFX_ONETIMEPAD_H__1E8EB5C2_2A48_11D7_A1AF_909CBD5DBF43__INCLUDED_)
