/*
//-----------------------------------------------------------------------------
OSS - OneTimePad V1.0 (C) 2003 by A. Ogut. All rights reserved. 

License to copy and use this software is granted provided that it 
is identified as the "OSS - OneTimePad" in all material mentioning 
or referencing this software or this function. 

License is also granted to make and use derivative works provided 
that such works are identified as "derived from the OSS - OneTimePad" 
in all material mentioning or referencing the derived work. 

OSS - Ogut Software Systems makes no representations concerning either 
the merchantability of this software or the suitability of this 
software for any particular purpose. It is provided "as is" 
without express or implied warranty of any kind. 

These notices must be retained in any copies of any part of this 
documentation and/or software. 
//-----------------------------------------------------------------------------
*/

// OneTimePadtl.c : One-Time-Pad Algorithm and Utility.
//
//-----------------------------------------------------------------------------

#include <stdlib.h>
#include <stdio.h>
#include "Longmath.h"
#include "Memalloc.h"
#include "md5.h"
#include "OneTimePadTl.h"
#include "AsciiLut.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//-----------------------------------------------------------------------------

void DWORD2FOURCC(DWORD dwData, BYTE *aByte)
{
   aByte[0] = (BYTE)((dwData >> 24) & 0xFF);
   aByte[1] = (BYTE)((dwData >> 16) & 0xFF);
   aByte[2] = (BYTE)((dwData >>  8) & 0xFF);
   aByte[3] = (BYTE) (dwData        & 0xFF);
}

//-----------------------------------------------------------------------------

DWORD FOURCC2DWORD(BYTE *aByte)
{
  DWORD dwData;

  dwData  = (DWORD)aByte[0] << 24;
  dwData |= (DWORD)aByte[1] << 16;
  dwData |= (DWORD)aByte[2] <<  8;
  dwData |= (DWORD)aByte[3];
  
  return dwData;
}

//-----------------------------------------------------------------------------
// Bits von DWORD durchmischen
DWORD MixDWORD(BOOL bForward, DWORD dwData, int nXchg, char *aXchg, char *bXchg)
{
  int dt = bForward ? +1 : -1;
  int i0 = bForward ?  0 : nXchg-1;
  int i,n,a,b;

  for (i = i0, n = 0; n < nXchg; n++, i += dt)
  {
    a = aXchg[i];
    b = bXchg[i];
    if (a != b)
    {
      DWORD aMask = ~(1 << a),
            bMask = ~(1 << b),
            aOr   = ((dwData >> a) & 1) << b,
            bOr   = ((dwData >> b) & 1) << a;

      dwData &= (aMask & bMask);
      dwData |= (aOr   | bOr);
    }
  
  } // for i,n

  return dwData;

}

//-----------------------------------------------------------------------------

void XORArray(int nData, char *aData, int nKey, char *aKey)
{
  int i, n = max(nData, nKey);
  
  for (i = 0; i < n; i++)
  {
    aData[i % nData] ^= aKey[i % nKey];

  } // for i
}

//-----------------------------------------------------------------------------

// Schlsseldatei erzeugen
// bAppend:    TRUE, wenn die Datei erweitert wird. FALSE, wenn die Datei neu erstellt werden soll.
// szKeyFile : Zeiger auf Dateinamen der Schlsseldatei
// nParam:     Anzahl der Operationen
// aParam:     Zeigerliste der Operationen (file:.., rnd:..)
// return:     TRUE, wenn ok, sonst FALSE bei Fehler
BOOL CreateKeyFile (BOOL bAppend, char *szKeyFile, int nParam, char *aParam[], ABORT_PROC_INFO *pAbortProcInfo)
{
  BOOL bBreak = FALSE, bSuccess = TRUE;
  char *buf = (char *)MALLOC(FILE_PAGESIZE);
  char *chParam;
  FILE *fp;
  int  i;

  if (buf != NULL)
  {
    if (bAppend) fp = fopen(szKeyFile, "r+b");
    else         fp = fopen(szKeyFile, "w+b");

    if (fp != NULL)
    {
      if (bAppend) fseek(fp, 0, SEEK_END);
      else         fseek(fp, 0, SEEK_SET);

      // Operationen anwenden
      for (i=0; i < nParam && !bBreak; i++)
      {
        if (!bSuccess) break;

        chParam = aParam[i];

        if (*chParam == 'r')
        {
          // Zufallszahlenfolge erstellen
          long rnd0, rnd, num, n, nb, nw;
          int  j;

          // Parameter einlesen
          sscanf(&chParam[5], "%lu %lu", &rnd0, &num);

          // Blockweise Zufallszahlen generieren und speichern
          for (n = 0; n < num && !bBreak; n+= FILE_PAGESIZE)
          {
            if (pAbortProcInfo != NULL)
            {
              ABORT_PROC_DATA Data;
              Data.lData[0] = i+1;
              Data.lData[1] = nParam;
              Data.lData[2] = (long)aParam[i];
              Data.lData[3] = n;
              Data.lData[4] = num;

              bBreak = pAbortProcInfo->pfAbortProc(pAbortProcInfo->lParam1, pAbortProcInfo->lParam2, &Data) == 1;
              if (bBreak) break;
            }

            nb = num - n; // Restlnge
            if (nb > FILE_PAGESIZE) nb = FILE_PAGESIZE;

            for (j=0; j < nb; j++)
              buf[j] = (char)LongRandom(&rnd, 256);
            
            // Block speichern
            nw  = fwrite(buf, 1, nb, fp);
            if (nw != nb) 
            {
              // Fehler aufgetreten!
              bSuccess = FALSE;
              break; // for n
            }
          
          } // for n

        } else 
        {
          // Datei hinzufgen
          FILE *fs;
          int  nRead, nWrite, nr;
          long n, fsize;

          fs = fopen(&chParam[6], "rb");
          if (fs != NULL)
          {
            // Dateilnge ermitteln
            fseek(fs, 0, SEEK_END);
            fsize = ftell(fs);
            fseek(fs, 0, SEEK_SET);
            n = 0;

            while (!feof(fs) && !bBreak)
            {
              if (pAbortProcInfo != NULL)
              {
                ABORT_PROC_DATA Data;
                Data.lData[0] = i+1;
                Data.lData[1] = nParam;
                Data.lData[2] = (long)aParam[i];
                Data.lData[3] = n;
                Data.lData[4] = fsize;

                bBreak = pAbortProcInfo->pfAbortProc(pAbortProcInfo->lParam1, pAbortProcInfo->lParam2, &Data) == 1;
                if (bBreak) 
                  break; // break while
              }

              // Datei kopieren
              nr = min(fsize - n, FILE_PAGESIZE);
              nRead  = fread(buf, 1, nr, fs); 
              if (nRead == 0) break; // break while

              nWrite = fwrite(buf, 1, nRead, fp);
              if (nRead != nWrite)
              {
                // Fehler aufgetreten!
                bSuccess = FALSE;
                break; // break while
              }
              n += nRead;

            } // while !feof(fs)
        
            fclose(fs);

          } else bSuccess = FALSE; // fs == NULL

        } // else Datei hinzufgen
      
      } // for i;

      fclose (fp);

    } else bSuccess = FALSE; // Fehler aufgetreten!

    FREE(buf);
  }

#if 0
  // TEST
  {
    STATINFO s;
    
    KeyFileStat (szKeyFile, &s, NULL);
  }
#endif

  return bSuccess;
}


//-----------------------------------------------------------------------------
// Schlsseldatei durchmischen
// szKeyFile: Zeiger auf Schlsseldatei
// nPass:     Anzahl der Durchlufe
// nWindowSize: Fenstergre in Bytes
// nWindowPass: Anzahl der Durchlufe innerhalb des Fensters
// rnd0:      Zeiger auf Zufallsvariable
// return : TRUE, wenn erfolgreich, sonst FALSE bei Fehler
BOOL MixKeyFile (char *szKeyFile, int nPass, long nWindowSize, int nWindowPass, long *rnd0, ABORT_PROC_INFO *pAbortProcInfo)
{
  BOOL bEof, bSuccess = TRUE, bBreak = FALSE;
  FILE *fp;
  long fpos, fsize, n;
  int  i, j, k, l, nRead, nWrite, nr;
  char *buf = (char *)MALLOC(nWindowSize);
  char  ch;

  if (buf != NULL)
  {
    fp = fopen(szKeyFile, "r+b");
    if (fp != NULL)
    {
      // Dateilnge ermitteln
      fseek(fp, 0, SEEK_END);
      fsize = ftell(fp);
      fseek(fp, 0, SEEK_SET);

      // mehrere Mischdurchlufe fhren
      for (i = 0; i < nPass && !bBreak; i++)
      {
        if (!bSuccess) break;

        // Dateizeiger an den Dateianfang setzen
        fseek(fp, 0, SEEK_SET);
        bEof = FALSE;

        n = 0;
        while (!bEof && !bBreak && bSuccess)
        {
          // Dateizeiger zum Wiederschreiben merken
          fpos = ftell(fp);

          if (pAbortProcInfo != NULL)
          {
            ABORT_PROC_DATA Data;
            Data.lData[0] = i+1;
            Data.lData[1] = nPass;
            Data.lData[2] = fpos;
            Data.lData[3] = fsize;
            Data.lData[4] = nWindowSize;
            Data.lData[5] = nWindowPass;
            Data.lData[6] = (long)szKeyFile;

            bBreak = pAbortProcInfo->pfAbortProc(pAbortProcInfo->lParam1, pAbortProcInfo->lParam2, &Data) == 1;
            if (bBreak)
              break; // break while
          }
        
          // Fenster lesen
          nr = min(fsize - n, nWindowSize);
          nRead  = fread(buf, 1, nr, fp);
          bEof   = feof(fp);
          if (nRead == 0) break; // keine Zeichen gelesen

          // Fenster durchmischen
          for (j = 0; j < nWindowPass; j++)
          {
            k = LongRandom(rnd0, nRead);
            l = LongRandom(rnd0, nRead);

            ch     = buf[k];
            buf[k] = buf[l];
            buf[l] = ch;
            
          } // for j (nWindowPass)

          // Fenster zurckschreiben
          fseek(fp, fpos, SEEK_SET);
          nWrite = fwrite(buf, 1, nRead, fp);
          if (nRead != nWrite)
          {
            // Fehler aufgetreten!
            bSuccess = FALSE;
            break;
          }
  
          n += nRead;

          if (!bEof)
            fseek(fp, fpos + nWindowSize/2, SEEK_SET);

        } // while (!bEof)

      } // for i (nPass)
  
      fclose(fp);
    
    } else bSuccess = FALSE; // fp ist NULL
    
    FREE(buf);

  } else bSuccess = FALSE; // buf ist NULL

#if 0
  // TEST
  {
    STATINFO s;
    
    KeyFileStat (szKeyFile, &s, NULL);
  }
#endif

  return bSuccess;
}


//-----------------------------------------------------------------------------
// Schlsseldatei erzeugen
// szKeyFile : Zeiger auf Dateinamen der Schlsseldatei
// nWindow:   Anzahl der Zeichen (maximal 256) im Fenster, in der 
//             keine Zeichenwiederholungen auftreten darf
// rnd0:      Zeiger auf Zufallsvariable
// return:    TRUE, wenn ok, sonst FALSE bei Fehler
BOOL RepeatLimiterKeyFile (char *szKeyFile, int nWindow, long *rnd0, ABORT_PROC_INFO *pAbortProcInfo)
{
  BOOL bEof, bSuccess = TRUE, bBreak = FALSE;
  FILE *fp;
  long fpos, fsize, n;
  int  i, j, nRead, nWrite, nr;
  enum { MAXWINDOW = 255 };
  char *buf = (char *)MALLOC(FILE_PAGESIZE);
  char chw[MAXWINDOW];
  char ch;

  if (buf != NULL)
  {
    nWindow = min(MAXWINDOW, nWindow);
    memset(chw, 0, nWindow);
    
    fp = fopen(szKeyFile, "r+b");
    if (fp != NULL)
    {
      // Dateilnge ermitteln
      fseek(fp, 0, SEEK_END);
      fsize = ftell(fp);
      fseek(fp, 0, SEEK_SET);

      bEof = FALSE;
      n = 0;
      while (!bEof && !bBreak && bSuccess)
      {
        // Dateizeiger zum Wiederschreiben merken
        fpos = ftell(fp);

        if (pAbortProcInfo != NULL)
        {
          ABORT_PROC_DATA Data;
          Data.lData[0] = fpos;
          Data.lData[1] = fsize;
          Data.lData[2] = nWindow;
          Data.lData[3] = (long)szKeyFile;

          bBreak = pAbortProcInfo->pfAbortProc(pAbortProcInfo->lParam1, pAbortProcInfo->lParam2, &Data) == 1;
          if (bBreak) 
            break; // break while
        }

        // Fenster lesen
        nr = min(fsize - n, FILE_PAGESIZE);
        nRead  = fread(buf, 1, nr, fp);
        bEof   = feof(fp);
        if (nRead == 0) break; // keine Zeichen gelesen

        // Fenster berprfen
        for (i = 0; i < nRead; i++)
        {
          ch = buf[i];

          do {

            // aktuelles Zeichen auf Vorhandensein im Fenster berprfen
            for (j = 0; j < nWindow; j++)
            {
              if (ch == chw[j]) 
                break; // Zeichen vorhanden

            } // for j
            
            if (j < nWindow) 
              ch = (char)LongRandom(rnd0, 256); // Zeichen ndern

          } while (j < nWindow);

          // Fenster verschieben und neues Zeichen in Fenster aufnehmen
          for (j = 0; j < nWindow-1; j++)
            chw[j] = chw[j+1];
          
          chw[nWindow-1] = ch;
          buf[i] = ch;
          
        } // for i (nRead)

        // Fenster zurckschreiben
        fseek(fp, fpos, SEEK_SET);
        nWrite = fwrite(buf, 1, nRead, fp);
        if (nRead != nWrite)
        {
          // Fehler aufgetreten!
          bSuccess = FALSE;
          break;
        }

        n += nRead;

      } // while (!bEof)

      fclose(fp);
    
    } else bSuccess = FALSE; // fp ist NULL

    FREE(buf);
    
  } else bSuccess = FALSE; // buf ist NULL

#if 0
  // TEST
  {
    STATINFO s;
    
    KeyFileStat (szKeyFile, &s, NULL);
  }
#endif

  return bSuccess;
}


//-----------------------------------------------------------------------------
// Statistik der Schlsseldatei erzeugen
// szKeyFile : Zeiger auf Dateinamen der Schlsseldatei
// pStat:      Zeiger auf berechnete Statistikdaten ber Daten (Rckgabe)
// pStatHisto: Zeiger auf berechnete Statistikdaten ber Histogramm (Rckgabe), darf NULL sein
// return:    TRUE, wenn ok, sonst FALSE bei Fehler
BOOL KeyFileStat (char *szKeyFile, STATINFO *pStat, STATINFO *pStatHisto)
{
  BOOL bSuccess = TRUE;
  FILE *fp;
  int  i, nRead, nr;
  long n, fsize;
  BYTE b;
  DWORD dwHisto[256];
  char *buf    = (char *)  MALLOC(FILE_PAGESIZE);
  double *xbuf = (double *)MALLOC(FILE_PAGESIZE * sizeof(double));
  double *ybuf = (double *)MALLOC(FILE_PAGESIZE * sizeof(double));

  if (buf != NULL && xbuf != NULL && ybuf != NULL)
  {
    //
    // Initialisieren
    //
    memset(dwHisto, 0, sizeof(dwHisto));
    StatisticsStart(pStat);
    
    fp = fopen(szKeyFile, "rb");
    if (fp != NULL)
    {
      // Dateilnge ermitteln
      fseek(fp, 0, SEEK_END);
      fsize = ftell(fp);
      fseek(fp, 0, SEEK_SET);
      
      n = 0;
      while (!feof(fp))
      {
        // Fenster lesen
        nr = min(fsize - n, FILE_PAGESIZE);
        nRead  = fread(buf, 1, nr, fp);
        if (nRead == 0) break; // keine Zeichen gelesen

        //
        // Statistik aktualisieren
        //
        for (i = 0; i < nRead; i++, n++)
        {
          xbuf[i] = n;
          ybuf[i] = (BYTE)buf[i];
        } // for i
        StatisticsUpdate(nRead, xbuf, ybuf, pStat);

        // Fenster auswerten
        for (i = 0; i < nRead; i++)
        {
          b = (BYTE)buf[i];
          if (dwHisto[b] != 0xffffffff) 
            dwHisto[b]++;

        } // for i (nRead)

      } // while (!feof)

      fclose(fp);

      //
      // Statistikergebnisse berechnen
      //
      StatisticsResult(pStat);

      //
      // Statistik ber Histogramm berechnen
      //
      if (pStatHisto != NULL)
      {
        double xbuf[256], ybuf[256];
        int i;

        for (i=0; i < 256; i++)
        {
          xbuf[i] = i;
          ybuf[i] = dwHisto[i];

        } // for i
    
        // Statistik ber Histogramm berechnen
        StatisticsStart(pStatHisto);
        //StatisticsUpdate(256, xbuf, ybuf, pStatHisto);
        StatisticsUpdate(256, ybuf, NULL, pStatHisto); // einfache Statistik
        StatisticsResult(pStatHisto);
      }

#if 0
      //TEST-Ausgabe Histogramm
      {
        char szBuf[1024];
        int i;

        OutputDebugString("Histogrammverteilung:\n");
        for (i=0; i < 256; i++)
        {
          sprintf(szBuf, "Histo[%d] = %lu\n", i, dwHisto[i]);
          OutputDebugString(szBuf);

        } // for i
      }
#endif
    
    } else bSuccess = FALSE; // fp ist NULL

    
  } else bSuccess = FALSE; // buf ist NULL


  // Aufrumen
  if (buf  != NULL) FREE(buf);
  if (xbuf != NULL) FREE(xbuf);
  if (ybuf != NULL) FREE(ybuf);

  return bSuccess;
}


//-----------------------------------------------------------------------------

typedef struct { // Verschlsselungs-Kontext
  BOOL          bEncode,      // TRUE zum Verschlsseln, FALSE zum Entschlsseln
                bConsumeKey;  // TRUE, wenn Schlssel nach dem Verwenden gelscht werden soll.
  unsigned char aIndex[4];    // Hashcode fr den Index auf Key-Stream

  KEYSTREAMINFO *pKeyInfo;    // Zeiger auf Schlsselinfo
  
  unsigned char md5[16];      // MD5-Code
  unsigned char mdt[16];      // MD5-Code (tmp)
  
  long          mdRndBitsA,   // Zufallszahlenvariablen fr Permutationen
                mdRndBitsB,
                mdRndkPosA,
                mdRndkPosB,
                mdRndMD5,
                mdRndLut[4];  
  
  BYTE          aLut[256],    // Look-Up-Tabelle fr Substitution
                aInvLut[256]; // Inverse Look-Up-Tabelle

  long          kPos;         // Zeiger auf Key-Stream-Position
                //kSize;        // Key-Streamlnge in Bytes

  FILE         *fKey;         // Dateizeiger der Schlsseldatei
  char         *pKey;         // Zeiger auf Schlsseldatenstream
  int           nPageSize,    // Seitengrsse fr Puffer
                nK,           // Zeiger auf pKey (Schlsselposition)
                nM,
                nB;

} CIPHERCONTEXT;


//-----------------------------------------------------------------------------
// Verschlsselungskontext initialisieren
// bEncode:    TRUE zum Verschlsseln, FALSE zum Entschlsseln
// szPassword: Zeiger auf Passwort
// pKeyInfo:   Zeiger auf Schlsselinformationen (Dateiname + Schlsselposition)
// pCC:        Zeiger auf Verschlsselungskontext (Rckgabe)
// return:     TRUE, wenn erfolgreich, sonst FALSE bei Fehler.
BOOL InitCipherContext(BOOL bEncode, char *szPassword, KEYSTREAMINFO *pKeyInfo, int nPageSize, CIPHERCONTEXT *pCC)
{
  DWORD         dw,
                A13DWSum, 
                B13DWSum,
                A13DWXor, 
                B13DWXor,
                rnd_a,
                rnd_b;

  MD5_CTX       md5_context;
  unsigned char md5[16];

  BYTE          b;
  int           i,j,m,n;

  memset(pCC, 0, sizeof(CIPHERCONTEXT));

  pCC->bEncode  = bEncode;
  pCC->pKeyInfo = pKeyInfo;

  //
  // MD5-Code generieren
  //
  MD5Init   (&md5_context); 
  MD5Update (&md5_context, (unsigned char *)szPassword, strlen (szPassword)); 
  MD5Final  (md5, &md5_context); 

  // Zufallszahlenvariablen initialisieren
  for (i=j=0; i < 4; i++,j += 4)
  {
    dw = ((DWORD)md5[j  ] << 24) |
         ((DWORD)md5[j+1] << 16) |
         ((DWORD)md5[j+2] <<  8) |
         ((DWORD)md5[j+3]      );
  
    pCC->mdRndLut[i] = (long)dw;
    
  } // for i

  for (i=0, A13DWXor = A13DWSum = 0; i < 13; i++)
  {
    dw = ((DWORD)md5[i  ] << 24) | 
         ((DWORD)md5[i+1] << 16) |
         ((DWORD)md5[i+2] <<  8) |
         ((DWORD)md5[i+3]      );
    
    A13DWXor ^= dw;
    A13DWSum += dw;

  } // for i

  for (i=0, B13DWXor = B13DWSum = 0; i < 13; i++)
  {
    dw = ((DWORD)md5[i+3] << 24) | 
         ((DWORD)md5[i+2] << 16) |
         ((DWORD)md5[i+1] <<  8) |
         ((DWORD)md5[i  ]      );
    
    B13DWXor ^= dw;
    B13DWSum += dw;

  } // for i

  //
  // Look-Up-Tabelle initialisieren
  //
  for (i = 0; i < 256; i++)
    pCC->aLut[i] = i;

  //
  // Permutierte Look-Up-Tabelle erstellen
  //
  rnd_a = A13DWSum xor A13DWXor;
  rnd_b = B13DWSum xor B13DWXor;
  for (i = 0; i < 16; i++)
  {
   for (j = 0; j < md5[i]; j++)
   {
     m = LongRandom(&rnd_a, 256);
     n = LongRandom(&rnd_b, 256);
     if (m != n)
     {
       b            = pCC->aLut[m];
       pCC->aLut[m] = pCC->aLut[n];
       pCC->aLut[n] = b;
     }

   } // for j

  } // for i

  // Inverse Look-Up-Tabelle bilden
  for (i = 0; i < 256; i++)
  {
    b = pCC->aLut[i];
    pCC->aInvLut[b] = i;
  }

  //
  // Daten bernehmen
  //
  memcpy(pCC->md5, md5, 16);
  memcpy(pCC->mdt, md5, 16);

  pCC->mdRndkPosA = pCC->mdRndLut[0] xor pCC->mdRndLut[3];
  pCC->mdRndkPosB = pCC->mdRndLut[1] xor pCC->mdRndLut[2];
  pCC->mdRndBitsA = A13DWXor;
  pCC->mdRndBitsB = B13DWXor;
  pCC->mdRndMD5   = A13DWXor xor B13DWXor;
  pCC->nB         = pCC->md5[0] & 63;
  pCC->kPos = pKeyInfo->nIndex;
  pCC->nPageSize = nPageSize;

  // Buffer fr Key-Stream reservieren und initialisieren
  pCC->pKey = (char *)MALLOC(nPageSize);
  if (pCC->pKey == NULL) return FALSE;
  memset(pCC->pKey, 0xFF, nPageSize);

  return TRUE;
}


//-----------------------------------------------------------------------------
// Schlsselindex ermitteln
// pCC:        Zeiger auf Verschlsselungskontext (Rckgabe)
// return:     TRUE, wenn erfolgreich, sonst FALSE bei Fehler.
BOOL InitCCFKey(CIPHERCONTEXT *pCC, BOOL bConsumeKey)
{
  int i,j;
  DWORD pos, rnd_a = pCC->mdRndkPosA, rnd_b = pCC->mdRndkPosB;

  // Index ver/entschlsseln  
  if (pCC->bEncode)
  { 
    // Index serialisieren
    DWORD2FOURCC(pCC->pKeyInfo->nIndex, pCC->aIndex);

    XORArray(4, pCC->aIndex, 16, pCC->md5);
  }

  pos = FOURCC2DWORD(pCC->aIndex);

  // Bit-Vertauschungen (Reihenfolge relevant!)
  {
    enum { MAX_XCHG = 16*2*16 }; // max 512
    char aX[MAX_XCHG],bX[MAX_XCHG];
    int  n,nX;
    
    for (i = nX = 0; i < 32; i++)
    {
      n = pCC->md5[i >> 1];
      if (i & 1) n >>= 4;
      else       n  &= 15;

      for (j = 0; j <= n; j++, nX++)
      {
        aX[nX] = (char)LongRandom(&rnd_a, 32);
        bX[nX] = (char)LongRandom(&rnd_b, 32);

      } // for j
    
    } // for i

    pos = MixDWORD(pCC->bEncode, pos, nX, aX, bX);
  }

  DWORD2FOURCC(pos, pCC->aIndex);

  if (!pCC->bEncode)
  {
    XORArray(4, pCC->aIndex, 16, pCC->md5);

    // Index rekonstruieren
    pCC->kPos = FOURCC2DWORD(pCC->aIndex);
  }

  // Key-Stream ffnen
  pCC->fKey = NULL;
  if (bConsumeKey) 
  {
    pCC->fKey = fopen(pCC->pKeyInfo->szKeyFile, "r+b"); // Key-Stream
    bConsumeKey = FALSE;
  }
  pCC->bConsumeKey = bConsumeKey;
  
  // Wenn read-write-Modus nicht mglich, dann read-only ffnen.
  if (pCC->fKey == NULL) 
    pCC->fKey = fopen(pCC->pKeyInfo->szKeyFile, "rb"); // Key-Stream

  if (pCC->fKey != NULL)
  {
    // Dateizeiger auf Schlsselposition setzen
    pCC->kPos = pCC->kPos % pCC->pKeyInfo->nSize;
    fseek(pCC->fKey, pCC->kPos, SEEK_SET);
  
  } else return FALSE;

  return TRUE;
}


//-----------------------------------------------------------------------------
// Schlsseldaten in Puffer einlesen
// pCC: Zeiger auf Verschlsselungskontext
BOOL CCFillupKeyStream(CIPHERCONTEXT *pCC)
{
  BOOL bSuccess = TRUE;
  int  nK, nRd;
  long fpos;

  //
  // Schlssel in Puffer laden
  //
  for (nK = 0; nK < pCC->nPageSize && bSuccess; /* none */)
  {
    fpos = ftell(pCC->fKey);
    nRd  = fread(&pCC->pKey[nK], 1, pCC->nPageSize - nK, pCC->fKey);
    if (nRd != 0) 
    {
      if (pCC->bConsumeKey)
      {
        // gelesene Schlssel nach dem Verbrauchen lschen
        char *pMem = (char *)MALLOC(nRd);
        int   nw;
        if (pMem != NULL)
        {
          memset(pMem, 0xff, nRd);
          fseek(pCC->fKey, fpos, SEEK_SET);
          nw = fwrite(pMem, 1, nRd, pCC->fKey);
          bSuccess = (nw == nRd);

          FREE(pMem);
        
        } else bSuccess = FALSE;
      }
      
      pCC->kPos += nRd;
      nK += nRd;
      if (nK < pCC->nPageSize) 
      {
        // Dateizeiger an den Anfang setzen und weiter lesen
        fseek(pCC->fKey, 0, SEEK_SET);
        pCC->kPos = 0;
      }
      
    } else 
    {
      // keine Daten gelesen
      bSuccess = FALSE;
      break;
    }

  } // for nK
  
  return bSuccess;
}


//-----------------------------------------------------------------------------
// Alphabet (LUT) viermal in Abhngigkeit von md5-Code permutieren
// pCC: Zeiger auf Verschlsselungskontext
void CCPermuteLut(CIPHERCONTEXT *pCC)
{
  BYTE tmp;
  int  i,m,n;

  for (i=0; i < 4; i++)
  {
    // Austauschkandidaten auswrfeln
    m = LongRandom(&pCC->mdRndLut[i],   256);  // i:   0 1 2 3
    n = LongRandom(&pCC->mdRndLut[3-i], 256);  // 3-i: 3 2 1 0
    if (m != n)
    {
      // Look-Up-Tabelle einmal permutieren (Substitution)
      tmp          = pCC->aLut[m];
      pCC->aLut[m] = pCC->aLut[n];
      pCC->aLut[n] = tmp;
    
      // Inverse LUT auch permutieren
      m = pCC->aLut[m];
      n = pCC->aLut[n];
      tmp             = pCC->aInvLut[m];
      pCC->aInvLut[m] = pCC->aInvLut[n];
      pCC->aInvLut[n] = tmp;
    }

  } // for i

}

//-----------------------------------------------------------------------------
// Datenbits mischen
// bEncode: TRUE->Vorwrts-Transformation, FALSE-> Rckwrts-Transformation
// Data:    Datum
// nPermut: Anzahl der Permutationen
// pCC:     Zeiger auf Verschlsselungskontext
// return:  Gemischte Datenbits
BYTE CCPermuteData(BOOL bEncode, BYTE Data, int nPermut, CIPHERCONTEXT *pCC)
{
  enum { MAXPERMUT = 4 };
  char xA[MAXPERMUT], xB[MAXPERMUT];
  int  i;

  //if (nPermut > MAXPERMUT) nPermut = MAXPERMUT;

  for (i=0; i < nPermut; i++)
  {
    xA[i] = (char)LongRandom(&pCC->mdRndBitsA, 8);
    xB[i] = (char)LongRandom(&pCC->mdRndBitsB, 8);
  }
  
  return (BYTE)MixDWORD(bEncode, Data, nPermut, xA, xB);

}

//-----------------------------------------------------------------------------
// Verschlsseln bzw. Entschlsseln
// bEncode: TRUE zum Verschlsseln, FALSE zum Entschlsseln
// nBuffer: Anzahl der Zeichen
// pSrc:    Zeiger auf Quelldaten
// pDst:    Zeiger auf Zieldaten
// pCC:     Zeiger auf Verschlsselungskontext
// return:  Anzahl verarbeiteter Zeichen (pDst)
int CCCipher(int nBuffer, char *pSrc, char *pDst, CIPHERCONTEXT *pCC)
{
  BYTE b, k, d, tmp;
  int  i, m, n;

  if (pCC->bEncode)
  {
    //
    // Daten verschlsseln
    //
    for (i = 0; i < nBuffer; i++)
    {
      b = (BYTE)pSrc[i];      // Quelldatum
      k = (BYTE)pCC->pKey[i]; // Schlssel (Pad)

      // hier ist das legendre "One-Time-Pad"
      d = b xor k; 

      // Datum-Bits mit MD5-Code verschlsseln
      d = d xor pCC->mdt[pCC->nM];

      // Datumbits mischen
      n = pCC->nB >> 2;
      m = (pCC->nB & 3) << 1;
      d = CCPermuteData(pCC->bEncode, d, ((pCC->mdt[n] >> m) & 3)+1, pCC);

      d = pCC->aLut[d];  // LUT-Substitution
      
      pDst[i] = d;       // Verschlsseltes Datum

      // MD5-Code einmal permutieren (in Abhngigkeit von MD5-Code)
      k = (BYTE)LongRandom(&pCC->mdRndMD5, 256);
      m = k >> 4;
      n = k & 15;
      tmp         = pCC->mdt[m];
      pCC->mdt[m] = pCC->mdt[n];
      pCC->mdt[n] = tmp;

      // Alphabet (LUT) viermal in Abhngigkeit von md5-Code permutieren
      CCPermuteLut(pCC);

      pCC->nM = (pCC->nM + 1) & 15;
      pCC->nB = (pCC->nB + 1) & 63;

    } // for i (nBuffer)
  } else 
  {
    //
    // Daten entschlsseln
    //
    for (i = 0; i < nBuffer; i++)
    {
      b = (BYTE)pSrc[i];      // Quelldatum
      k = (BYTE)pCC->pKey[i]; // Schlssel (Pad)

      b = pCC->aInvLut[b];    // Inverse LUT-Substitution

      // Datumbits entmischen
      n = pCC->nB >> 2;
      m = (pCC->nB & 3) << 1;
      b = CCPermuteData(pCC->bEncode, b, ((pCC->mdt[n] >> m) & 3)+1, pCC);
      
      // hier ist das legendre "One-Time-Pad"
      d = b xor k;

      // Datum-Bits mit MD5-Code verschlsseln
      d = d xor pCC->mdt[pCC->nM];

      pDst[i] = d; // Entschlsseltes Datum

      // MD5-Code einmal permutieren (in Abhngigkeit von MD5-Code)
      k = (BYTE)LongRandom(&pCC->mdRndMD5, 256);
      m = k >> 4;
      n = k & 15;
      tmp         = pCC->mdt[m];
      pCC->mdt[m] = pCC->mdt[n];
      pCC->mdt[n] = tmp;

      // Alphabet (LUT) viermal in Abhngigkeit von md5-Code permutieren
      CCPermuteLut(pCC);

      pCC->nM = (pCC->nM + 1) & 15;
      pCC->nB = (pCC->nB + 1) & 63;

    } // for i (nBuffer)
  }
  
  return nBuffer;
}


//-----------------------------------------------------------------------------
// Resourcen des Verschlsselungskontextes freigeben
// pCC: Zeiger auf Verschlsselungskontext
void CloseCipherContext(CIPHERCONTEXT *pCC)
{
  if (pCC->bEncode)
  {
    // kPos zurckschreiben
    pCC->pKeyInfo->nIndex = pCC->kPos;
  }

  if (pCC->fKey != NULL) fclose(pCC->fKey);
  if (pCC->pKey != NULL) FREE(pCC->pKey);
  memset(pCC, 0, sizeof(CIPHERCONTEXT));
}


//-----------------------------------------------------------------------------
// Binrdatei ver- bzw. entschlsseln
// bEncode:    TRUE zum Verschlsseln, FALSE zum Entschlsseln
// bConsumeKey: TRUE, wenn Schlssel verbraucht werden.
// szPassword: Zeiger auf Passwort
// szFileSrc:  Zeiger auf Quelldateiname
// szFileDst:  Zeiger auf Zieldateiname
// pKeyInfo:   Zeiger auf Schlsselinformationen (Dateiname + Schlsselposition)
// return:     TRUE, wenn erfolgreich, sonst FALSE bei Fehler.
BOOL EncodeFile (BOOL bEncode, BOOL bConsumeKey, char *szPassword, char *szFileSrc, char *szFileDst, KEYSTREAMINFO *pKeyInfo)
{
  BOOL bSuccess = TRUE;
  FILE *fd, *fs;
  long n, fsize;
  int  nRd, nWr, nr;
  char *pSrc = (char *)MALLOC(FILE_PAGESIZE);
  char *pDst = (char *)MALLOC(FILE_PAGESIZE);
  CIPHERCONTEXT CC;

  fs = fopen(szFileSrc, "rb"); // Data-Stream   (Source)
  fd = fopen(szFileDst, "w+b");// Output-Stream (Destination)
  
  if (fs   != NULL && fd   != NULL && 
      pSrc != NULL && pDst != NULL)
  {
    // Quelldateilnge ermitteln
    fseek(fs, 0, SEEK_END);
    fsize = ftell(fs);
    fseek(fs, 0, SEEK_SET);

    //
    // Verschlsselungsvariable initialisieren
    //
    bSuccess = InitCipherContext(bEncode, szPassword, pKeyInfo, FILE_PAGESIZE, &CC);

    //
    // Index laden und entschlsseln
    //
    if (!bEncode && bSuccess)
    {
      //
      // Index laden und entschlsseln
      //
      nRd  = fread(CC.aIndex, 1, 4, fs);
      if (nRd != 4) 
        bSuccess = FALSE; // Fehler!

    }

    if (bSuccess) 
      bSuccess &= InitCCFKey(&CC, bConsumeKey); // Index ver-/entschlsseln

    if (bEncode && bSuccess)
    { 
      // Zieldaten speichern
      nWr = fwrite(CC.aIndex, 1, 4, fd);
      if (nWr != 4)
        bSuccess = FALSE; // Fehler!
    }

    //
    // Ver(-/ent)schlsselung beginnen
    //
    n = 0;
    while (!feof(fs) && bSuccess)
    {
      //
      // Schlsselstream einlesen
      //
      CCFillupKeyStream(&CC);

      //
      // Quelldaten laden
      //
      nr = min(fsize - n, FILE_PAGESIZE);
      nRd  = fread(pSrc, 1, nr, fs);
      if (nRd == 0) break; // keine Zeichen gelesen

      //
      // ver- bzw. entschlsseln
      //
      CCCipher(nRd, pSrc, pDst, &CC);

      // Zieldaten speichern
      nWr = fwrite(pDst, 1, nRd, fd);
      if (nRd != nWr)
      {
        // Fehler aufgetreten!
        bSuccess = FALSE;
        break; // while
      }
      
      n += nRd;

    } // while (!bEof)

    CloseCipherContext(&CC);
    
  } else bSuccess = FALSE; // Fehler ist aufgetreten!

  //
  // Aufrumen
  //
  if (fs != NULL) fclose(fs);
  if (fd != NULL) fclose(fd);
  
  if (pSrc != NULL) FREE(pSrc);
  if (pDst != NULL) FREE(pDst);

  return bSuccess;
}


//-----------------------------------------------------------------------------
// Puffer ver- bzw. entschlsseln
// bEncode: TRUE zum Verschlsseln, FALSE zum Entschlsseln
// bConsumeKey: TRUE, wenn Schlssel verbraucht werden.
// szPassword: Zeiger auf Passwort
// nBuffer:    Anzahl Zeichen im Buffer pSrc
// pSrc:       Zeiger auf zu ver- bzw. entschlsselnde Daten
// pDst:       Zeiger auf Ergebnisbuffer
// szKeyInfo:  Zeiger auf Schlsselinformationen
// return:     Anzahl Zeichen im pDst;
int EncodeBuffer (BOOL bEncode, BOOL bConsumeKey, char *szPassword, int nBuffer, char *pSrc, char *pDst, KEYSTREAMINFO *pKeyInfo)
{
  CIPHERCONTEXT CC;
  BOOL bSuccess;
  int  nRet;

  if (!bEncode) nBuffer -= 4;
  nRet = nBuffer;
  
  //
  // Verschlsselungsvariable initialisieren
  //
  bSuccess = InitCipherContext(bEncode, szPassword, pKeyInfo, nBuffer, &CC);

  //
  // Index laden und entschlsseln
  //
  if (!bEncode && bSuccess)
  {
    // Index entschlsseln
    memcpy(CC.aIndex, pSrc, 4);
    pSrc += 4;
  }

  if (bSuccess)
    bSuccess &= InitCCFKey(&CC, bConsumeKey); // Index ver-/entschlsseln

  if (bEncode && bSuccess)
  { 
    // verschlsselten Index schreiben
    memcpy(pDst, CC.aIndex, 4);
    pDst += 4; 
    nRet += 4;
  }

  //
  // Schlsselstream einlesen
  //
  if (bSuccess) 
    bSuccess &= CCFillupKeyStream(&CC);

  //
  // ver- bzw. entschlsseln
  //
  if (bSuccess) 
    bSuccess &= CCCipher(nBuffer, pSrc, pDst, &CC);

  //
  // Aufrumen
  //
  CloseCipherContext(&CC);

  return nRet;
}


//-----------------------------------------------------------------------------
// Konvertiert Binrdatei in eine 6-Bit-ASCII Datei um
// szFileSrc: Zeiger auf Quelldateiname
// szFileDst: Zeiger auf Zieldateiname
// return: TRUE, wenn erfolgreich, sonst FALSE
BOOL ConvertBin2ASCII (char *szFileSrc, char *szFileDst)
{ 
  BOOL bSuccess = TRUE;
  FILE *fs, *fd;
  long fsize, n;
  int  i, nRead, nWrite, nWr, nOut, nr;

  //Aus 3-Byte werden 4 Zeichen
  enum { ASCII_PAGESIZE = 3000 //-> 4000 Zeichen Buffer notwendig
       };
  char *bufSrc   = (char *)MALLOC( ASCII_PAGESIZE);
  char *bufTmp   = (char *)MALLOC((ASCII_PAGESIZE*4)/3);
  char *bufDst   = (char *)MALLOC((ASCII_PAGESIZE*4)/3);
  char *aLut     = GetStatic6BitLut();

  fs = fopen(szFileSrc, "rb");
  fd = fopen(szFileDst, "w+t");

  if (bufSrc != NULL && bufDst != NULL && bufTmp != NULL &&
          fs != NULL && fd != NULL)
  {
    // Quelldateilnge ermitteln
    fseek(fs, 0, SEEK_END);
    fsize = ftell(fs);
    fseek(fs, 0, SEEK_SET);
    
    n = 0;
    while (!feof(fs) && bSuccess)
    {
      // Fenster lesen
      nr = min(fsize - n, ASCII_PAGESIZE);
      nRead  = fread(bufSrc, 1, nr, fs);
      if (nRead == 0) break; // keine Zeichen gelesen

      // Fenster umwandeln

      // 8 Bit -> 6 Bit Code umwandeln
      nOut = Convert8to6Bit(nRead, bufSrc, bufTmp);
      Convert6toASCII(aLut, nOut, bufTmp, bufDst);
      
      // ASCII-Codes schreiben (max MAX_PER_LINE_ASCII (40) Zeichen pro Zeile)
      for (i = 0; i < nOut; i += MAX_PER_LINE_ASCII)
      {
        nWr = nOut - i;
        nWr = min(nWr, MAX_PER_LINE_ASCII);
        nWrite = fwrite(&bufDst[i], 1, nWr, fd);
        if (nWr != nWrite)
        {
          // Fehler aufgetreten!
          bSuccess = FALSE;
          break; // break for
        }
        fputs("\n", fd); // CR/LF Einfgen
        
      } 

      n += nRead;

    } // while (!feof)
   
  } else bSuccess = FALSE; // Fehler aufgetreten

  //
  // Aufrumen
  //
  if (fs != NULL) fclose(fs);
  if (fd != NULL) fclose(fd);

  if (bufSrc != NULL) FREE(bufSrc);
  if (bufTmp != NULL) FREE(bufTmp);
  if (bufDst != NULL) FREE(bufDst);
  
  return bSuccess;
}


//-----------------------------------------------------------------------------
// Konvertiert 6-Bit-ASCII Date in eine Binrdatei um
// szFileSrc: Zeiger auf Quelldateiname
// szFileDst: Zeiger auf Zieldateiname
// return: TRUE, wenn erfolgreich, sonst FALSE
BOOL ConvertASCII2Bin (char *szFileSrc, char *szFileDst)
{ 
  BOOL bSuccess = TRUE;
  FILE *fs, *fd;
  int  i, nRead, nWrite, nWr, nCodes, nOfs, nr;
  long fsize, n;

  //Aus 4 Zeichen werden 3 Byte
  enum { ASCII_PAGESIZE = 4000 };
  char *bufSrc   = (char *)MALLOC(ASCII_PAGESIZE);
  char *bufTmp   = (char *)MALLOC(ASCII_PAGESIZE);
  char *bufDst   = (char *)MALLOC((ASCII_PAGESIZE*3)/4);
  char *aLut     = GetStaticInvLut();

  fs = fopen(szFileSrc, "rb");
  fd = fopen(szFileDst, "w+b");

  if (bufSrc != NULL && bufDst != NULL && bufTmp != NULL &&
          fs != NULL && fd != NULL)
  {
    // Quelldateilnge ermitteln
    fseek(fs, 0, SEEK_END);
    fsize = ftell(fs);
    fseek(fs, 0, SEEK_SET);
    
    for (n = 0, nOfs = 0; bSuccess; n += nRead)
    {
      // Fenster lesen
      nr = fsize - n;
      nRead = ASCII_PAGESIZE - nOfs;
      nr = min(nr, nRead);

      nRead  = fread(bufSrc, 1, nr, fs);
      if (nRead == 0 && nOfs == 0)
        break; // Ende erreicht

      // neue 6 Bit ASCII -> 6 Bit binr Code umwandeln
      nCodes = ConvertASCIIto6(aLut, nRead, bufSrc, &bufTmp[nOfs]);

      // Restcodes dazunehmen
      nCodes += nOfs;
      
      if (nRead != 0)
      {
        nOfs  = nCodes & 3; // vierer Gruppen kodieren - Rest merken
        nCodes -= nOfs;     // ohne Rest Codes kodieren
      
      } else nOfs = 0;
      
      // 6 Bit Code -> 8 Bit char umwandeln
      nWr = Convert6to8Bit(nCodes, bufTmp, bufDst);

      // Restcodes an den Anfang kopieren
      for (i = 0; i < nOfs; i++)
        bufTmp[i] = bufTmp[i + nCodes];

      // Umgewandelte Codes schreiben
      nWrite = fwrite(bufDst, 1, nWr, fd);
      if (nWr != nWrite)
      {
        // Fehler aufgetreten!
        bSuccess = FALSE;
        break; // break for
      }
      
    } // for
   
  } else bSuccess = FALSE; // Fehler aufgetreten!

  //
  // Aufrumen
  //
  if (fs != NULL) fclose(fs);
  if (fd != NULL) fclose(fd);

  if (bufSrc != NULL) FREE(bufSrc);
  if (bufTmp != NULL) FREE(bufTmp);
  if (bufDst != NULL) FREE(bufDst);
  
  return bSuccess;
}


//-----------------------------------------------------------------------------
// Ver/Entschlsselt eine Datei in 6-Bit ASCII-Datei.
// bEncode: TRUE zum Verschlsseln, FALSE zum Entschlsseln
// bConsumeKey: TRUE, wenn Schlssel verbraucht werden.
// szPassword: Zeiger auf Passwort
// szFileSrc: Zeiger auf Quelldateiname
// szFileDst: Zeiger auf Zieldateiname
// szKeyInfo:  Zeiger auf Schlsselinformationen
// return: TRUE, wenn erfolgreich, sonst FALSE
BOOL EncodeASCIIFile (BOOL bEncode, BOOL bConsumeKey, char *szPassword, char *szFileSrc, char *szFileDst, KEYSTREAMINFO *pKeyInfo)
{
  BOOL bRet = FALSE;
  char *pTmp;

  pTmp = _tempnam( P_tmpdir, "~1tap" );
  if (pTmp != NULL) 
  {
    if (bEncode)
    {
      if ( EncodeFile (bEncode, bConsumeKey, szPassword, szFileSrc, pTmp, pKeyInfo) )
      {
        if ( ConvertBin2ASCII (pTmp, szFileDst) )
          bRet = TRUE;
      }
    } else
    {
      if ( ConvertASCII2Bin (szFileSrc, pTmp) )
      {
        if ( EncodeFile (bEncode, bConsumeKey, szPassword, pTmp, szFileDst, pKeyInfo) )
          bRet = TRUE;
      }
    }

    // Tmp-Datei nach Gebrauch wieder lschen
    if (remove( pTmp) == -1)
      bRet = FALSE;
 
    free(pTmp);
  }
  
  return bRet;
}


//-----------------------------------------------------------------------------
// Lscht eine Datei
// szFile: Zeiger auf Dateinamen
// bEraseTotal: TRUE, wenn die Datei komplett unleserlich gelscht werden soll
// return: TRUE, wenn erfolgreich
BOOL EraseFile(char *szFile, BOOL bEraseTotal)
{
  BOOL bRet = TRUE;

  if (bEraseTotal) // Datei "zerstren"
  {
    FILE *fp;
    long fsize, p;
    int  nWrite,n;
    char *buf = (char *)MALLOC(FILE_PAGESIZE);

    if (buf != NULL)
    {
      // Buffer mit leeren Werten fllen
      memset(buf, 0x55, FILE_PAGESIZE);

      fp = fopen(szFile, "r+b");
      if (fp != NULL)
      {
        // Dateilnge ermitteln
        fseek(fp, 0, SEEK_END);
        fsize = ftell(fp);
        fseek(fp, 0, SEEK_SET);

        // Datei seitenweise mit leeren Werten fllen
        for (p=0; p < fsize; p += FILE_PAGESIZE)
        {
          n = min(FILE_PAGESIZE, fsize - p);

          nWrite = fwrite(buf, 1, n, fp);
          if (nWrite != n)
          {
            // Fehler aufgetreten!
            bRet = FALSE;
            break; // break for
          }
        } // for p

        fclose(fp);

      } else bRet = FALSE; // fp == NULL
      
      FREE(buf);

    } else bRet = FALSE; // buf == NULL

  }
  
  // Datei lschen
  if (remove(szFile) == -1)
    bRet = FALSE;

  return bRet;
}


//-----------------------------------------------------------------------------
// Ermittelt die Dateilnge in Bytes
// szFile: Zeiger auf Dateiname
// bRW:    optional: RW-Zugriff mglich? TRUE, wenn Schreibzugriff mglich, sonst FALSE;
// return: Dateilnge in Bytes
long FileSize(char *szFile, BOOL *bRW)
{
  FILE *fp;
  long fsize = 0;

  if (bRW != NULL) 
    *bRW = FALSE;

  fp = fopen(szFile, "rb");

  if (fp != NULL)
  {
    // Dateilngen ermitteln
    fseek(fp, 0, SEEK_END);
    fsize = ftell(fp);
    fclose(fp);

    if (bRW != NULL) 
    {
      // Schreibzugriff mglich?
      fp = fopen(szFile, "r+b");
      *bRW = (fp != NULL);
      if (fp != NULL) 
        fclose(fp);
    }
  } 
  
  return fsize;
}


//-----------------------------------------------------------------------------
// Vergleicht zwei Dateien auf exakte bereinstimmung
// szFile1 : Zeiger auf Dateiname 1
// szFile2 : Zeiger auf Dateiname 2
// return:   TRUE, wenn beide Dateien exakt gleich sind, FALSE wenn verschieden
BOOL FileCompare(char *szFile1, char *szFile2)
{
  BOOL bSuccess = FALSE;
  FILE *fp1, *fp2;
  long fsize1, fsize2, n;
  int  i, nRead1, nRead2, nr;
  char *buf1 = (char *)MALLOC(FILE_PAGESIZE);
  char *buf2 = (char *)MALLOC(FILE_PAGESIZE);

  fp1 = fopen(szFile1, "rb");
  fp2 = fopen(szFile2, "rb");

  if (fp1  != NULL && fp2  != NULL &&
      buf1 != NULL && buf2 != NULL)
  {
    // Dateilngen ermitteln
    fseek(fp1, 0, SEEK_END);
    fsize1 = ftell(fp1);
    fseek(fp1, 0, SEEK_SET);

    fseek(fp2, 0, SEEK_END);
    fsize2 = ftell(fp2);
    fseek(fp2, 0, SEEK_SET);

    bSuccess = (fsize1 == fsize2);

    if (bSuccess)
    {
      // Dateien inhaltlich vergleichen
      for (n=0; n < fsize1 && bSuccess; n+= nRead1)
      {
        // Fenster lesen
        nr = min(fsize1 - n, FILE_PAGESIZE);
        nRead1 = fread(buf1, 1, nr, fp1);
        nRead2 = fread(buf2, 1, nr, fp2);

        bSuccess &= (nRead1 == nRead2);
        if (!bSuccess) break; // nicht gleich!
        if (nRead1 == 0) break; // Ende erreicht

        // Fenster berprfen
        for (i = 0; i < nRead1 && bSuccess; i++)
        {
          bSuccess = (buf1[i] == buf2[i]);
        } // for i

      } // for n

    }
    
  }

  if (buf1 != NULL) FREE(buf1);
  if (buf2 != NULL) FREE(buf2);
  if (fp1  != NULL) fclose(fp1);
  if (fp2  != NULL) fclose(fp2);

  return bSuccess;
}


//-----------------------------------------------------------------------------
// Testet alle Kodierungs -und Dekodierungsalgorithmen
// szFile: Quelldatei, welche kodiert, dekodiert und verglichen wird
// szPwd:  Password zur Verschlsselung
// pKeyInfo: Zeiger auf Schlsselinformationen
// return:  TRUE, wenn der Test erfolgreich war, sonst FALSE
BOOL TestCiphering(char *szFile, char *szPwd, KEYSTREAMINFO *pKeyInfo)
{
  BOOL bRet = FALSE;
  char *pTmp1,
       *pTmp2;

  // Temporre Dateinamen erstellen
  pTmp1 = _tempnam( P_tmpdir, "~1tp_tmp1" );
  pTmp2 = _tempnam( P_tmpdir, "~1tp_tmp2" );
  if (pTmp1 != NULL && pTmp2 != NULL) 
  {
    bRet = TRUE;

    // Test1: Binr verschlsseln/entschlsseln/vergleichen
    if (bRet) 
    {
      if (bRet) bRet &= EncodeFile (TRUE,  FALSE, szPwd, szFile, pTmp1, pKeyInfo);
      if (bRet) bRet &= EncodeFile (FALSE, FALSE, szPwd, pTmp1,  pTmp2, pKeyInfo);
      if (bRet) bRet &= FileCompare(szFile, pTmp2);
      EraseFile(pTmp1, FALSE);
      EraseFile(pTmp2, FALSE);
    }

    // Test2: Binr in ASCII verschlsseln/entschlsseln/vergleichen
    if (bRet) 
    {
      if (bRet) bRet &= EncodeASCIIFile (TRUE,  FALSE, szPwd, szFile, pTmp1, pKeyInfo);
      if (bRet) bRet &= EncodeASCIIFile (FALSE, FALSE, szPwd, pTmp1,  pTmp2, pKeyInfo);
      if (bRet) bRet &= FileCompare(szFile, pTmp2);
      EraseFile(pTmp1, FALSE);
      EraseFile(pTmp2, FALSE);
    }

    // Test3: ASCII kodieren/dekodieren/vergleichen
    if (bRet) 
    {
      if (bRet) bRet &= ConvertBin2ASCII(szFile, pTmp1);
      if (bRet) bRet &= ConvertASCII2Bin(pTmp1,  pTmp2);
      if (bRet) bRet &= FileCompare(szFile, pTmp2);
      EraseFile(pTmp1, FALSE);
      EraseFile(pTmp2, FALSE);
    }

  }

  // Tmp-Dateien nach Gebrauch wieder lschen
  if (pTmp1 != NULL) { remove(pTmp1); free(pTmp1); }
  if (pTmp2 != NULL) { remove(pTmp2); free(pTmp2); }
  
  return bRet;
}


//-----------------------------------------------------------------------------
//todo.....
BOOL cmpBits(BYTE *aBits, DWORD nSize, BYTE bCmp, DWORD *dwSumBits)
{ 
  static const int aSumBits[16] = 
  //   0    1    2    3    4    5    6    7    8    9    a    b    c    d    e    f
  //0000 0001 0010 0011 0100 0101 0110 0111 1000 1001 1010 1011 1100 1101 1110 1111
   {0,   1,   1,   2,   1,   2,   2,   3,   1,   2,   2,   3,   2,   3,   3,   4 };
  DWORD n, nBits;
  BYTE  b;
  BOOL  bRet = TRUE;
  
  for (n = nBits = 0; n < nSize; n++)
  {
    b = aBits[n];
    nBits += aSumBits[b & 15];
    nBits += aSumBits[b >> 4];

    if (b != bCmp)
      bRet = FALSE;
  }
  *dwSumBits = nBits;
  return bRet;
}


BOOL TestRandomGenerator(long *rnd0, DWORD min, DWORD max)
{
  enum {WINDOW_SIZE = 1024*1024*16, WINDOW_BYTES = WINDOW_SIZE/8};
  
  BYTE *aRndBits = (BYTE *)MALLOC(WINDOW_BYTES);
  DWORD i, n, m, adr, nTrials = WINDOW_SIZE, nBits;
  BYTE  bit;
  char szBuf[256];
  BOOL bAbort = FALSE, bReady;
  HWND hwndAbort;

  if (aRndBits == NULL) return FALSE;
  min = 0;
  max = WINDOW_SIZE;
  memset(aRndBits, 0, WINDOW_BYTES);
  
  hwndAbort  = AbortInstall(NULL, FALSE, "Random Test", "Random Test");
  AbortDlgSetSize(hwndAbort, 400, 150);

  do
  {
    bReady = cmpBits(aRndBits, WINDOW_BYTES, 0xFF, &nBits);
    if (!bReady)
    {
      for (i=m=0; i < nTrials && !bAbort; m++/*none*/)
      {
        n = (DWORD) LongRnd(rnd0);
        if (n >= min && n < max)
        {
          n -= min;
          bit = (BYTE)(n & 7);
          adr = n >> 3;
          aRndBits[adr] |= 1 << bit;

          i++; // Treffer zhlen

          if ((i % 1000) == 0)
          {
            bReady = cmpBits(aRndBits, WINDOW_BYTES, 0xFF, &nBits);

            sprintf(szBuf, "Random-Test: [%lu - %lu[\n%lu Treffer von %lu\nnBits = %lu of %lu", min, max, i, m, nBits, WINDOW_SIZE);
            AbortSetText(hwndAbort, szBuf);
            
            bAbort = Abort(hwndAbort);
          }

        }

      } // for i
    
    }
      
  } while (!bAbort && !bReady);
  
  AbortRemove(hwndAbort);

  FREE(aRndBits);
  return TRUE;

}

