/*
//-----------------------------------------------------------------------------
OSS - OneTimePad V1.0 (C) 2003 by A. Ogut. All rights reserved. 

License to copy and use this software is granted provided that it 
is identified as the "OSS - OneTimePad" in all material mentioning 
or referencing this software or this function. 

License is also granted to make and use derivative works provided 
that such works are identified as "derived from the OSS - OneTimePad" 
in all material mentioning or referencing the derived work. 

OSS - Ogut Software Systems makes no representations concerning either 
the merchantability of this software or the suitability of this 
software for any particular purpose. It is provided "as is" 
without express or implied warranty of any kind. 

These notices must be retained in any copies of any part of this 
documentation and/or software. 
//-----------------------------------------------------------------------------
*/

// OneTimePadtl.h : One-Tap-Algorithm and Utility.

#ifndef __INC_ONETIMEPADTL__
#define __INC_ONETIMEPADTL__

#include <windows.h>
#include "DlgAbort.h"
#include "Statistics.h"

#ifdef __cplusplus
extern "C" { // Assume C declarations for C++
#endif       // __cplusplus

//-----------------------------------------------------------------------------------

//
// Konstanten fr die Anwendung
//
enum { 
  MAX_PER_LINE_ASCII = 40 // 40 Zeichen pro Zeile
};

typedef struct {
  BOOL bRW;                  // RW-Modus mglich?

  long nIndex,               // aktuelle Schlsselposition
       nSize;                // Schlssellnge in Bytes

  char szKeyFile[_MAX_PATH]; // Schlsseldateiname

} KEYSTREAMINFO;

BOOL CreateKeyFile (BOOL bAppend, char *szKeyFile, int nParam, char *aParam[], ABORT_PROC_INFO *pAbortProcInfo);
BOOL MixKeyFile    (char *szKeyFile, int nPass, long nWindowSize, int nWindowPass, long *rnd0, ABORT_PROC_INFO *pAbortProcInfo);
BOOL RepeatLimiterKeyFile (char *szKeyFile, int nWindow, long *rnd0, ABORT_PROC_INFO *pAbortProcInfo);
BOOL KeyFileStat   (char *szKeyFile, STATINFO *pStat, STATINFO *pStatHisto);
BOOL EncodeFile    (BOOL bEncode, BOOL bConsumeKey, char *szPassword, char *szFileSrc, char *szFileDst, KEYSTREAMINFO *pKeyInfo);
int  EncodeBuffer  (BOOL bEncode, BOOL bConsumeKey, char *szPassword, int nBuffer, char *pSrc, char *pDst, KEYSTREAMINFO *pKeyInfo);
BOOL EncodeASCIIFile (BOOL bEncode, BOOL bConsumeKey, char *szPassword, char *szFileSrc, char *szFileDst, KEYSTREAMINFO *pKeyInfo);

BOOL ConvertBin2ASCII (char *szFileSrc, char *szFileDst);
BOOL ConvertASCII2Bin (char *szFileSrc, char *szFileDst);

BOOL EraseFile(char *szFile, BOOL bEraseTotal);
long FileSize (char *szFile, BOOL *bRW);
BOOL FileCompare(char *szFile1, char *szFile2);

//TEST
BOOL TestRandomGenerator(long *rnd0, DWORD min, DWORD max);
BOOL TestCiphering(char *szFile, char *szPwd, KEYSTREAMINFO *pKeyInfo);


//-----------------------------------------------------------------------------------
#ifdef __cplusplus
}       // End of extern "C" {
#endif  // __cplusplus

//-----------------------------------------------------------------------------------

#endif  //ifndef __INC_ONETIMEPADTL__
