/*
//-----------------------------------------------------------------------------
OSS - OneTimePad V1.0 (C) 2003 by A. Ogut. All rights reserved. 

License to copy and use this software is granted provided that it 
is identified as the "OSS - OneTimePad" in all material mentioning 
or referencing this software or this function. 

License is also granted to make and use derivative works provided 
that such works are identified as "derived from the OSS - OneTimePad" 
in all material mentioning or referencing the derived work. 

OSS - Ogut Software Systems makes no representations concerning either 
the merchantability of this software or the suitability of this 
software for any particular purpose. It is provided "as is" 
without express or implied warranty of any kind. 

These notices must be retained in any copies of any part of this 
documentation and/or software. 
//-----------------------------------------------------------------------------
*/

/* Statistics.h */

#ifndef __INC_STATISTICS__
#define __INC_STATISTICS__

#include <math.h>

#ifdef __cplusplus
extern "C" { // Assume C declarations for C++
#endif       // __cplusplus

//-----------------------------------------------------------------------------------

typedef struct {
  int      N;        // Anzahl der Daten

  double   XMin,     // Minimum fr X
           XMax,     // Maximum fr X
           XSum,     // Summe der X
           XXSum,    // Summe der X*X
           YMin,     // Minimum fr Y
           YMax,     // Maximum fr Y
           YSum,     // Summe der Y
           YYSum,    // Summe der Y*Y
           XYSum,    // Summe der X*Y
           XM,       // Mittelwert X (XSum/N)
           YM,       // Mittelwert Y (YSum/N)
           StdX,     // Standardabweichung (fr X = sqrt(VarX))
           StdY,     // Standardabweichung (fr Y = sqrt(VarY))
           VarX,     // Varianz X
           VarY,     // Varianz Y
           Kov,      // Kovarianz
           Korr,     // Korrelationsfaktor
           M,        // Steigung der Geraden
           T;        // y-Achsabschnitt

} STATINFO;

int  Statistics(int N, double *BufX, double *BufY, STATINFO *pStatInfo);
void LinRegInt (int Anz, int *BufX, int *BufY, double *Kov, double *VarX, double *VarY, double *xm, double *ym);

void StatisticsStart (STATINFO *pStatInfo);
int  StatisticsUpdate(int N, double *BufX, double *BufY, STATINFO *pStatInfo);
int  StatisticsResult(STATINFO *pStatInfo);

int  StatisticsXY    (int N, double *BufX, double *BufY, STATINFO *pStatInfo);
int  StatisticsHisto (int nBuf, DWORD *dwBuf, STATINFO *pStatInfo);


int  MessageBoxStatisticsXY(HWND hWnd, LPCTSTR lpszCaption, STATINFO *pStatInfo);
int  MessageBoxStatisticsX (HWND hWnd, LPCTSTR lpszCaption, STATINFO *pStatInfo);

#ifdef _DEBUG
void DebugOutStatistics(STATINFO *pStatInfo);
#else
#define DebugOutStatistics(s)
#endif


//-----------------------------------------------------------------------------------
#ifdef __cplusplus
}       // End of extern "C" {
#endif  // __cplusplus

//-----------------------------------------------------------------------------------

#endif  //ifndef __INC_STATISTICS__
